@@ -2,6 +2,7 @@
 ### CompressedList objects
 ### -------------------------------------------------------------------------
 
+
 setClass("CompressedList",
          contains="List",
          representation(
@@ -11,6 +12,7 @@ setClass("CompressedList",
                        )
          )
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Accessor methods.
 ###
@@ -35,6 +37,7 @@ setReplaceMethod("names", "CompressedList",
                      x
                  })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor.
 ###
@@ -44,11 +47,72 @@ setReplaceMethod("names", "CompressedList",
 ### when calling this from another package.
 ###
 
+compress_listData <- function(x) {
+    if (length(x) > 0L) {
+        if (length(dim(x[[1L]])) < 2L) {
+            x <- do.call(c, unname(x))
+        } else {
+            x <- do.call(rbind, unname(x))
+        }
+    } else {
+        x <- vector()
+    }
+    x
+}
+
+.reconcileMetadatacols <- function(x) {
+  x_mcols <- mcols(x)
+  if (is(x_mcols, "DataFrame") &&
+      nrow(x_mcols) == 0L && ncol(x_mcols) == 0L)
+    {
+      x_mcols <- new("DataFrame", nrows=length(x))
+      mcols(x) <- x_mcols
+    }
+  x
+}
+
+### Low-level. NOT exported.
 newCompressedList0 <- function(Class, unlistData, partitioning)
 {
     ans <- new2(Class, unlistData=unlistData,
                 partitioning=partitioning, check=FALSE)
-    reconcileMetadatacols(ans)
+    .reconcileMetadatacols(ans)
+}
+
+### Low-level. NOT exported.
+### Stuff to put in elementMetadata slot can be passed either with
+###   new_CompressedList_from_list(..., elementMetadata=somestuff)
+### or with
+###   new_CompressedList_from_list(..., mcols=somestuff)
+### The latter is the new recommended form.
+new_CompressedList_from_list <- function(Class, x, ..., mcols)
+{
+    if (!extends(Class, "CompressedList"))
+        stop("class ", Class, " must extend CompressedList")
+    if (!is.list(x))
+        stop("'x' must be a list")
+    ans_elementType <- elementType(new(Class))
+    if (!all(sapply(x, function(xi) extends(class(xi), ans_elementType))))
+        stop("all elements in 'listData' must be ", ans_elementType, " objects")
+    ans_partitioning <- PartitioningByEnd(x)
+    if (length(x) == 0L) {
+        if (missing(mcols))
+            return(new2(Class, partitioning=ans_partitioning, ..., check=FALSE))
+        return(new2(Class, partitioning=ans_partitioning, ...,
+                    elementMetadata=mcols, check=FALSE))
+    }
+    ans_unlistData <- compress_listData(x)
+    if (missing(mcols)) {
+        ans <- new2(Class, unlistData=ans_unlistData,
+                    partitioning=ans_partitioning, ...,
+                    check=FALSE)
+    } else {
+        ans <- new2(Class, unlistData=ans_unlistData,
+                    partitioning=ans_partitioning, ...,
+                    elementMetadata=mcols,
+                    check=FALSE)
+    }
+    .reconcileMetadatacols(ans)
 }
 
 
@@ -80,6 +144,84 @@ newCompressedList0 <- function(Class, unlistData, partitioning)
 setValidity2("CompressedList", .valid.CompressedList)
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Coercion.
+###
+
+setUnlistDataNames <- function(unlisted_x, grouping, use.names, x_class)
+{
+    ## If 'use.names' is FALSE or 'x' has no *outer* names, then we don't
+    ## do anything to 'ans' i.e. we just keep whatever names/rownames are
+    ## on it (which are the *inner* names/rownames of 'x'). Note that this
+    ## behavior is NOT consistent with unlist,List or base::unlist as
+    ## both of them will return a vector with no names/rownames when
+    ## 'use.names' is FALSE.
+    ## FIXME: Make unlist,CompressedList and unlist,List behave
+    ## consistently in *any* situation.
+    ## Otherwise (i.e. if 'use.names' is TRUE and 'x' has *outer* names),
+    ## we make up new names/rownames for 'ans' by prepending the *outer*
+    ## names of 'x' to its *inner* names/rownames. Note that this differs
+    ## from what base::unlist does but THIS IS A FEATURE and is consistent
+    ## with what unlist,List does.
+    if (use.names && !is.null(x_names <- names(grouping))) {
+        if (length(dim(unlisted_x)) < 2L) {
+            ans_ROWNAMES <- names(unlisted_x)
+        } else {
+            ans_ROWNAMES <- rownames(unlisted_x)
+        }
+        nms <- rep.int(x_names, elementLengths(grouping))
+        ans_ROWNAMES <- S4Vectors:::make_unlist_result_names(nms, ans_ROWNAMES)
+        if (length(dim(unlisted_x)) < 2L) {
+            res <- try(names(unlisted_x) <- ans_ROWNAMES, silent=TRUE)
+            what <- "names"
+        } else {
+            res <- try(rownames(unlisted_x) <- ans_ROWNAMES, silent=TRUE)
+            what <- "rownames"
+        }
+        if (is(res, "try-error"))
+            warning("failed to set ", what, " on the ",
+                    "unlisted ", x_class, " object")
+    }
+    unlisted_x
+}
+
+setMethod("unlist", "CompressedList",
+    function(x, recursive=TRUE, use.names=TRUE)
+    {
+        if (!identical(recursive, TRUE))
+            stop("\"unlist\" method for CompressedList objects ",
+                 "does not support the 'recursive' argument")
+        if (!isTRUEorFALSE(use.names))
+            stop("'use.names' must be TRUE or FALSE")
+        setUnlistDataNames(x@unlistData, x@partitioning, use.names, class(x))
+    }
+)
+
+setAs("ANY", "CompressedList", function(from) coerceToCompressedList(from))
+
+coerceToCompressedList <- function(from, element.type = NULL, ...) {
+  if (is(from, S4Vectors:::listClassName("Compressed", element.type)))
+    return(from)
+  if (is.list(from) || (is(from, "List") && !is(from, "DataFrame"))) {
+    if (is.list(from)) {
+      v <- compress_listData(from)
+    } else {
+      v <- unlist(from, use.names = FALSE)
+    }
+    part <- PartitioningByEnd(from)
+  } else {
+    v <- from
+    part <- PartitioningByEnd(seq_len(NROW(from)))
+  }
+  if (!is.null(element.type)) {
+    v <- S4Vectors:::coercerToClass(element.type)(v, ...)
+  }
+  to <- relist(v, part)
+  names(to) <- names(from)
+  to
+}
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
 ###
@@ -87,13 +229,21 @@ setValidity2("CompressedList", .valid.CompressedList)
 setMethod("extractROWS", "CompressedList",
     function(x, i)
     {
-        if (missing(i) || !is(i, "Ranges"))
-            i <- normalizeSingleBracketSubscript(i, x)
-        ir <- IRanges(end=extractROWS(end(x@partitioning), i),
-                      width=extractROWS(width(x@partitioning), i))
-        ans_unlistData <- extractROWS(x@unlistData, ir)
+        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+        ans_eltlens <- extractROWS(width(x@partitioning), i)
+        ans_breakpoints <- suppressWarnings(cumsum(ans_eltlens))
+        nbreakpoints <- length(ans_breakpoints)
+        if (nbreakpoints != 0L && is.na(ans_breakpoints[[nbreakpoints]]))
+            stop(wmsg("Subsetting operation on ", class(x), " object 'x' ",
+                      "produces a result that is too big to be ",
+                      "represented as a CompressedList object. ",
+                      "Please try to coerce 'x' to a SimpleList object ",
+                      "first (with 'as(x, \"SimpleList\")')."))
+        idx_on_unlisted_x <- IRanges(end=extractROWS(end(x@partitioning), i),
+                                     width=ans_eltlens)
+        ans_unlistData <- extractROWS(x@unlistData, idx_on_unlisted_x)
         ans_partitioning <- new2("PartitioningByEnd",
-                                 end=cumsum(width(ir)),
+                                 end=ans_breakpoints,
                                  NAMES=extractROWS(names(x), i),
                                  check=FALSE)
         ans_elementMetadata <- extractROWS(x@elementMetadata, i)
@@ -112,8 +262,9 @@ setMethod("getListElement", "CompressedList",
             return(NULL)
         unlisted_x <- unlist(x, use.names=FALSE)
         x_partitioning <- PartitioningByEnd(x)
-        i <- IRanges(start(x_partitioning)[i], end(x_partitioning)[i])
-        extractROWS(unlisted_x, i)
+        window_start <- start(x_partitioning)[i]
+        window_end <- end(x_partitioning)[i]
+        window.Vector(unlisted_x, start=window_start, end=window_end)
     }
 )
 
@@ -121,7 +272,7 @@ setReplaceMethod("[[", "CompressedList",
                  function(x, i, j,..., value)
                  {
                      nameValue <- if (is.character(i)) i else ""
-                     i <- normargSubset2_iOnly(x, i, j, ...,
+                     i <- S4Vectors:::normargSubset2_iOnly(x, i, j, ...,
                               .conditionPrefix="[[<-,CompressedList-method: ")
                      if (is.null(value)) {
                          if (i <= length(x)) # if name did not exist, could be +1
@@ -151,7 +302,7 @@ setReplaceMethod("[[", "CompressedList",
                            new2("PartitioningByEnd", end = cumsum(widths),
                                 NAMES = NAMES, check=FALSE)
                          if (i > length(x))
-                           x <- rbindRowOfNAsToMetadatacols(x)
+                           x <- S4Vectors:::rbindRowOfNAsToMetadatacols(x)
                          x
                      }
                  })
@@ -164,7 +315,7 @@ setReplaceMethod("$", "CompressedList",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining and splitting.
+### Combining.
 ###
 
 .bindROWS <- function(...)
@@ -181,7 +332,7 @@ unlist_list_of_CompressedList <- function(x)
     ans_unlistData <- do.call(.bindROWS, lapply(x, slot, "unlistData"))
     ans_eltlens <- unlist(lapply(x, elementLengths))
     ans <- relist(ans_unlistData, PartitioningByEnd(cumsum(ans_eltlens)))
-    ans_mcols <- do.call(rbind.mcols, x)
+    ans_mcols <- do.call(S4Vectors:::rbind_mcols, x)
     rownames(ans_mcols) <- NULL
     mcols(ans) <- ans_mcols
     ans
@@ -209,6 +360,7 @@ setMethod("c", "CompressedList",
               unlist_list_of_CompressedList(tls)
           })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Looping.
 ###
@@ -238,16 +390,16 @@ lapply_CompressedList <- function(X, FUN, ...)
         return(ans)
     X_elt_start <- start(X_partitioning)
     X_elt_end <- end(X_partitioning)
-    old_validity_status <- disableValidity()
-    disableValidity(TRUE)
-    on.exit(disableValidity(old_validity_status))
+    old_validity_status <- S4Vectors:::disableValidity()
+    S4Vectors:::disableValidity(TRUE)
+    on.exit(S4Vectors:::disableValidity(old_validity_status))
     ans[non_empty_idx] <-
       lapply(non_empty_idx,
              function(i)
                  FUN(extractROWS(unlisted_X,
                                  IRanges(X_elt_start[i], X_elt_end[i])),
                      ...))
-    disableValidity(old_validity_status)
+    S4Vectors:::disableValidity(old_validity_status)
     for (i in non_empty_idx) {
         obj <- ans[[i]]
         if (isS4(obj) && !isTRUE(validObject(obj, test=TRUE)))
@@ -265,29 +417,6 @@ setMethod("lapply", "CompressedList",
     }
 )
 
-setMethod("aggregate", "CompressedList",
-          function(x, by, FUN, start = NULL, end = NULL, width = NULL,
-                   frequency = NULL, delta = NULL, ..., simplify = TRUE)
-          {
-              if (!missing(by) && is(by, "RangesList")) {
-                  if (length(x) != length(by))
-                      stop("for Ranges 'by', 'length(x) != length(by)'")
-                  y <- as.list(x)
-                  result <-
-                    lapply(structure(seq_len(length(x)), names = names(x)),
-                           function(i)
-                               aggregate(y[[i]], by = by[[i]], FUN = FUN,
-                                         frequency = frequency, delta = delta,
-                                         ..., simplify = simplify))
-                  ans <- try(SimpleAtomicList(result), silent = TRUE)
-                  if (inherits(ans, "try-error"))
-                      ans <- newList("SimpleList", result)
-              } else {
-                  ans <- callNextMethod()
-              }
-              ans
-          })
-
 .updateCompressedList <- function(X, listData) {
     elementTypeX <- elementType(X)
     if (!all(sapply(listData,
@@ -321,87 +450,25 @@ setMethod("mendoapply", "CompressedList",
 setMethod("revElements", "CompressedList",
     function(x, i)
     {
-        i <- normalizeSingleBracketSubscript(i, x)
-        if (length(x) == 0L)
+        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+        if (length(i) == 0L)
             return(x)
         elt_lens <- elementLengths(x)
         offset <- cumsum(c(0L, elt_lens[-length(elt_lens)]))
         rev <- logical(length(x))
-        rev[i] <- TRUE
-        ii <- fancy_mseq(elt_lens, offset=offset, rev=rev)
-        x@unlistData <- x@unlistData[ii]
+        rev <- replaceROWS(rev, i, TRUE)
+        ii <- S4Vectors:::fancy_mseq(elt_lens, offset=offset, rev=rev)
+        x@unlistData <- extractROWS(x@unlistData, ii)
         x
     }
 )
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion.
+### classNameForDisplay()
 ###
 
-setMethod("unlist", "CompressedList",
-    function(x, recursive=TRUE, use.names=TRUE)
-    {
-        if (!identical(recursive, TRUE))
-            stop("\"unlist\" method for CompressedList objects ",
-                 "does not support the 'recursive' argument")
-        if (!isTRUEorFALSE(use.names))
-            stop("'use.names' must be TRUE or FALSE")
-        ans <- x@unlistData
-        ## If 'use.names' is FALSE or 'x' has no *outer* names, then we don't
-        ## do anything to 'ans' i.e. we just keep whatever names/rownames are
-        ## on it (which are the *inner* names/rownames of 'x'). Note that this
-        ## behavior is NOT consistent with unlist,List or base::unlist as
-        ## both of them will return a vector with no names/rownames when
-        ## 'use.names' is FALSE.
-        ## FIXME: Make unlist,CompressedList and unlist,List behave
-        ## consistently in *any* situation.
-        ## Otherwise (i.e. if 'use.names' is TRUE and 'x' has *outer* names),
-        ## we make up new names/rownames for 'ans' by prepending the *outer*
-        ## names of 'x' to its *inner* names/rownames. Note that this differs
-        ## from what base::unlist does but THIS IS A FEATURE and is consistent
-        ## with what unlist,List does.
-        if (use.names && !is.null(x_names <- names(x))) {
-            if (length(dim(ans)) < 2L) {
-                ans_ROWNAMES <- names(ans)
-            } else {
-                ans_ROWNAMES <- rownames(ans)
-            }
-            nms <- rep.int(x_names, elementLengths(x))
-            ans_ROWNAMES <- make.unlist.result.names(nms, ans_ROWNAMES)
-            if (length(dim(ans)) < 2L) {
-                res <- try(names(ans) <- ans_ROWNAMES, silent=TRUE)
-                what <- "names"
-            } else {
-                res <- try(rownames(ans) <- ans_ROWNAMES, silent=TRUE)
-                what <- "rownames"
-            }
-            if (is(res, "try-error"))
-                warning("failed to set ", what, " on the result ",
-                        "of unlisting a ", class(x), " object")
-        }
-        ans
-    }
+setMethod("classNameForDisplay", "CompressedList",
+    function(x) sub("^Compressed", "", class(x))
 )
 
-coerceToCompressedList <- function(from, element.type = NULL, ...) {
-  if (is(from, listClassName("Compressed", element.type)))
-    return(from)
-  if (is.list(from) || is(from, "List")) {
-    if (is.list(from)) {
-      v <- compress_listData(from)
-    } else {
-      v <- unlist(from, use.names = FALSE)
-    }
-    part <- PartitioningByEnd(from)
-  } else {
-    v <- from
-    part <- PartitioningByEnd(seq_len(length(from)))
-  }
-  if (!is.null(element.type)) {
-    v <- coercerToClass(element.type)(v, ...)
-  }
-  to <- relist(v, part)
-  names(to) <- names(from)
-  to
-}
