@@ -38,7 +38,7 @@ setReplaceMethod("active", "FilterRules", function(x, value) {
     nfilters <- length(x)
     if (length(value) > nfilters)
       stop("length of 'value' must not be greater than that of 'filters'")
-    if (anyMissing(value))
+    if (S4Vectors:::anyMissing(value))
       stop("'value' cannot contain NA's")
     if (nfilters && (nfilters %% length(value) != 0))
       stop("number of filters not a multiple of 'value' length")
@@ -80,14 +80,15 @@ FilterRules <- function(exprs = list(), ..., active = TRUE) {
 
   active <- rep(active, length.out = length(exprs))
 
-  if (!is.logical(active) || anyMissing(active))
+  if (!is.logical(active) || S4Vectors:::anyMissing(active))
     stop("'active' must be logical without any missing values")
   if (length(active) > length(exprs))
     stop("length of 'active' is greater than number of rules")
   if (length(exprs) && length(exprs) %% length(active) > 0)
     stop("number of rules must be a multiple of length of 'active'")
 
-  ans <- newList("FilterRules", exprs, active = active)
+  ans <- S4Vectors:::new_SimpleList_from_list("FilterRules", exprs,
+                                              active = active)
   validObject(ans)
   ans
 }
@@ -137,7 +138,7 @@ setMethod("[", "FilterRules",
     "length of 'active' must match length of 'filters'"
   else if (!identical(names(active(x)), names(x)))
     "names of 'active' must match those of 'filters'"
-  else if (anyMissing(active(x)))
+  else if (S4Vectors:::anyMissing(active(x)))
     "'active' cannot contain NA's"
   else NULL
 }
@@ -374,7 +375,7 @@ FilterMatrix <- function(matrix, filterRules) {
 
 setMethod("show", "FilterMatrix", function(object) {
   cat(class(object), " (", nrow(object), " x ", ncol(object), ")\n", sep = "")
-  mat <- makePrettyMatrixForCompactPrinting(object, function(x) x@.Data)
+  mat <- S4Vectors:::makePrettyMatrixForCompactPrinting(object, function(x) x@.Data)
   print(mat, quote = FALSE, right = TRUE)
 })
 
