@@ -2,11 +2,33 @@
 ### Grouping objects
 ### -------------------------------------------------------------------------
 ###
-### We call "grouping" the action of dividing a collection of NO objects into
-### NG groups (some of them eventually empty). The Grouping class and
-### subclasses are containers for representing groupings.
+### We call "grouping" an arbitrary mapping from a collection of NO objects
+### to a collection of NG groups, or, more formally, a bipartite graph
+### between integer sets [1, NO] and [1, NG]. Objects mapped to a given group
+### are said to belong to, or to be assigned to, or to be in that group.
+### Additionally, the objects in each group are ordered. So for example
+### the 2 following groupings are considered different:
+###
+###   Grouping 1: NG = 3, NO = 5
+###               group   objects
+###                   1 : 4, 2
+###                   2 :
+###                   3 : 4
+###
+###   Grouping 2: NG = 3, NO = 5
+###               group   objects
+###                   1 : 2, 4
+###                   2 :
+###                   3 : 4
+###
+### There are no restriction on the mapping e.g. any object can be mapped
+### to 0, 1, or more groups, and can be mapped twice to the same group. Also
+### some or all the groups can be empty.
 ###
 
+### The Grouping class is a virtual class that formalizes the most general
+### kind of grouping. More specific groupings (e.g. many-to-one mappings)
+### are formalized via specific Grouping subclasses.
 setClass("Grouping", contains="IntegerList", representation("VIRTUAL"))
 
 setGeneric("nobj", function(x) standardGeneric("nobj"))
@@ -24,11 +46,37 @@ setMethod("grouplength", "Grouping",
     }
 )
 
+setMethod("show", "Grouping",
+    function(object)
+    {
+        NG <- length(object)
+        NO <- nobj(object)
+        cat(class(object), " with ",
+            NG, ifelse(NG == 1, " group ", " groups "),
+            "and ", NO, ifelse(NO == 1, " object\n", " objects\n"),
+            sep="")
+        if (NG == 0L)
+            return(invisible(NULL))
+        empty_groups <- which(grouplength(object) == 0L)
+        cat("Nb of empty groups: ", length(empty_groups),
+            " (", 100.00 * length(empty_groups) / NG, "%)\n", sep="")
+    }
+)
+
+
+### -------------------------------------------------------------------------
+### ManyToOneGrouping objects
+### -------------------------
+
+### A ManyToOneGrouping object represents a grouping where every object in
+### the collection is mapped to one group and only one.
+setClass("ManyToOneGrouping", contains="Grouping", representation("VIRTUAL"))
+
 setGeneric("members", signature="x",
     function(x, i) standardGeneric("members")
 )
 
-setMethod("members", "Grouping",
+setMethod("members", "ManyToOneGrouping",
     function(x, i)
     {
         if (!is.numeric(i))
@@ -43,7 +91,7 @@ setGeneric("vmembers", signature="x",
     function(x, L) standardGeneric("vmembers")
 )
 
-setMethod("vmembers", "Grouping",
+setMethod("vmembers", "ManyToOneGrouping",
     function(x, L)
     {
         if (!is.list(L))
@@ -70,7 +118,7 @@ setMethod("togroup", "ANY",
         if (!is.integer(j))
             j <- as.integer(j)
         bound <- length(to_group)
-        if (anyMissingOrOutside(j, -bound, bound))
+        if (S4Vectors:::anyMissingOrOutside(j, -bound, bound))
             stop("subscript 'j' contains NAs or out of bounds indices")
         to_group[j]
     }
@@ -80,36 +128,18 @@ setGeneric("togrouplength", signature="x",
     function(x, j=NULL) standardGeneric("togrouplength")
 )
 
-setMethod("togrouplength", "Grouping",
+setMethod("togrouplength", "ManyToOneGrouping",
     function(x, j=NULL)
         grouplength(x, togroup(x, j))
 )
 
-setMethod("show", "Grouping",
-    function(object)
-    {
-        NG <- length(object)
-        NO <- nobj(object)
-        cat(class(object), " with ",
-            NG, ifelse(NG == 1, " group ", " groups "),
-            "and ", NO, ifelse(NO == 1, " object\n", " objects\n"),
-            sep="")
-        if (NG == 0L)
-            return(invisible(NULL))
-        empty_groups <- which(grouplength(object) == 0L)
-        cat("Nb of empty groups: ", length(empty_groups),
-            " (", 100.00 * length(empty_groups) / NG, "%)\n", sep="")
-    }
-)
-
-
 
 ### -------------------------------------------------------------------------
 ### BiIndexGrouping objects
 ### -----------------------
 
 #setClass("BiIndexGrouping",
-#    contains="Grouping",
+#    contains="ManyToOneGrouping",
 #    representation(
 #        group2object="list",
 #        object2group="integer"
@@ -126,11 +156,11 @@ setMethod("show", "Grouping",
 ### H2LGrouping and Dups objects
 ### ----------------------------
 ###
-### High-to-Low Index Grouping objects.
+### High-to-Low Index ManyToOneGrouping objects.
 ###
 
 setClass("H2LGrouping",
-    contains="Grouping",
+    contains="ManyToOneGrouping",
     representation(
         high2low="integer",
         low2high="list"
@@ -163,7 +193,8 @@ setMethod("getListElement", "H2LGrouping",
     }
 )
 
-### Should be more efficient than the default method for Grouping objects.
+### Should be more efficient than the default method for ManyToOneGrouping
+### objects.
 setMethod("grouplength", "H2LGrouping",
     function(x, i=NULL)
     {
@@ -176,7 +207,7 @@ setMethod("grouplength", "H2LGrouping",
         if (!is.integer(i))
             i <- as.integer(i)
         bound <- length(group_length)
-        if (anyMissingOrOutside(i, -bound, bound))
+        if (S4Vectors:::anyMissingOrOutside(i, -bound, bound))
             stop("subscript 'i' contains NAs or out of bounds indices")
         group_length[i]
     }
@@ -215,7 +246,7 @@ setMethod("togroup", "H2LGrouping",
         if (!is.integer(j))
             j <- as.integer(j)
         bound <- length(to_group)
-        if (anyMissingOrOutside(j, -bound, bound))
+        if (S4Vectors:::anyMissingOrOutside(j, -bound, bound))
             stop("subscript 'j' contains NAs or out of bounds indices")
         to_group[j]
     }
@@ -238,7 +269,7 @@ setMethod("togroup", "H2LGrouping",
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### More operations on H2LGrouping objects. These operations are NOT part of
-### the Grouping core API.
+### the core Grouping API.
 ###
 
 ### The rank of group G_i is the number of non-empty groups that are before
@@ -280,14 +311,6 @@ setMethod("togrouprank", "H2LGrouping",
     }
 )
 
-makeLow2highFromHigh2low <- function(high2low)
-{
-    ans <- vector(mode="list", length=length(high2low))
-    sparse_ans <- split(seq_along(high2low), high2low)
-    ans[as.integer(names(sparse_ans))] <- sparse_ans
-    ans
-}
-
 setReplaceMethod("length", "H2LGrouping",
     function(x, value)
     {
@@ -300,7 +323,7 @@ setReplaceMethod("length", "H2LGrouping",
         if (value > length(x))
             stop("cannot make a ", class(x), " instance longer")
         length(x@high2low) <- value
-        x@low2high <- makeLow2highFromHigh2low(x@high2low)
+        x@low2high <- S4Vectors:::reverseSelfmatchMapping(x@high2low)
         x
     }
 )
@@ -330,7 +353,9 @@ setReplaceMethod("length", "H2LGrouping",
         return("the 'low2high' slot must contain a list")
     if (length(x@high2low) != length(x@low2high))
         return("the 'high2low' and 'low2high' slots must have the same length")
-    if (!identical(makeLow2highFromHigh2low(x@high2low), x@low2high)) {
+    if (!identical(S4Vectors:::reverseSelfmatchMapping(x@high2low),
+                   x@low2high))
+    {
         problem <- c("the 'low2high' slot must contain the reverse mapping ",
                      "of the 'high2low' slot")
         return(paste(problem, collapse=""))
@@ -381,7 +406,7 @@ setMethod("show", "Dups",
     if (!is.integer(high2low))
         high2low <- as.integer(high2low)
     new2(Class, high2low=high2low,
-         low2high=makeLow2highFromHigh2low(high2low),
+         low2high=S4Vectors:::reverseSelfmatchMapping(high2low),
          check=FALSE)
 }
 
@@ -458,15 +483,15 @@ setMethod("high2low", "Vector",
 ### starting at 1 (i.e. it covers an integer interval starting at 1
 ### and with no overlap between the ranges). Therefore the "start/end/width"
 ### API is implemented on Partitioning objects (in addition to the Grouping
-### API).
+### and ManyToOneGrouping APIs).
 ###
-### The Partitioning class is virtual with 2 concrete subclasses:
-### PartitioningByEnd and PartitioningByWidth.
-### Note that we put Ranges before Grouping in order to have Partitioning
-### objects inherit the "show" method for Ranges objects.
+### The Partitioning class is virtual with 3 concrete subclasses:
+### PartitioningByEnd and PartitioningByWidth and PartitioningMap.
+### Note that we put Ranges before ManyToOneGrouping in order to have
+### Partitioning objects inherit the "show" method for Ranges objects.
 
 setClass("Partitioning",
-    contains=c("Ranges", "Grouping"),
+    contains=c("Ranges", "ManyToOneGrouping"),
     representation(
         "VIRTUAL",
         NAMES="characterORNULL"  # R doesn't like @names !!
@@ -499,7 +524,8 @@ setMethod("getListElement", "Partitioning",
     }
 )
 
-### Should be more efficient than the default method for Grouping objects.
+### Should be more efficient than the default method for ManyToOneGrouping
+### objects.
 setMethod("grouplength", "Partitioning",
     function(x, i=NULL)
     {
@@ -511,7 +537,7 @@ setMethod("grouplength", "Partitioning",
         if (!is.integer(i))
             i <- as.integer(i)
         bound <- length(x_width)
-        if (anyMissingOrOutside(i, -bound, bound))
+        if (S4Vectors:::anyMissingOrOutside(i, -bound, bound))
             stop("subscript 'i' contains NAs or out of bounds indices")
         x_width[i]
     }
@@ -583,7 +609,9 @@ setMethod("start", "PartitioningByEnd",
     }
 )
 
-setMethod("width", "PartitioningByEnd", function(x) diffWithInitialZero(end(x)))
+setMethod("width", "PartitioningByEnd",
+    function(x) S4Vectors:::diffWithInitialZero(end(x))
+)
 
 .valid.PartitioningByEnd <- function(x)
 {
@@ -591,9 +619,9 @@ setMethod("width", "PartitioningByEnd", function(x) diffWithInitialZero(end(x)))
         return("the ends must be integers")
     if (length(x) == 0L)
         return(NULL)
-    if (anyMissing(end(x)))
+    if (S4Vectors:::anyMissing(end(x)))
         return("the ends cannot be NAs")
-    if (isNotSorted(end(x)))
+    if (S4Vectors:::isNotSorted(end(x)))
         return("the ends must be sorted")
     if (end(x)[1L] < 0L)
         return("the ends cannot be negative")
@@ -623,12 +651,19 @@ PartitioningByEnd <- function(x=integer(), NG=NULL, names=NULL)
         x <- cumsum(elementLengths(x))
     } else {
         if (!is.numeric(x))
-            stop("'x' must be either a list-like object ",
-                 "or a sorted integer vector")
+            stop("'x' must be either a list-like object or ",
+                 "a sorted vector of non-NA non-negative integers")
         if (!is.integer(x))
             x <- as.integer(x)
-        if (isNotSorted(x))
-            stop("when 'x' is an integer vector, it must be sorted")
+        if (any(is.na(x)))
+            stop("when 'x' is an integer vector, ",
+                 "it cannot contain NAs")
+        if (length(x) != 0L && min(x) < 0L)
+            stop("when 'x' is an integer vector, ",
+                 "it cannot contain negative values")
+        if (S4Vectors:::isNotSorted(x))
+            stop("when 'x' is an integer vector, ",
+                 "it must be sorted")
         if (!is.null(NG)) {
             ## When 'NG' (number of groups) is supplied, then 'x' is considered
             ## to represent the group assignment of a collection of 'length(x)'
@@ -726,7 +761,7 @@ setMethod("start", "PartitioningByWidth",
         return("the widths must be integers")
     if (length(x) == 0L)
         return(NULL)
-    if (anyMissing(width(x)))
+    if (S4Vectors:::anyMissing(width(x)))
         return("the widths cannot be NAs")
     if (any(width(x) < 0L))
         return("the widths cannot be negative")
@@ -746,9 +781,16 @@ PartitioningByWidth <- function(x=integer(), NG=NULL, names=NULL)
         x <- elementLengths(x)
     } else {
         if (!is.numeric(x))
-            stop("'x' must be either a list-like object or an integer vector")
+            stop("'x' must be either a list-like object or ",
+                 "a vector of non-NA non-negative integers")
         if (!is.integer(x))
             x <- as.integer(x)
+        if (any(is.na(x)))
+            stop("when 'x' is an integer vector, ",
+                 "it cannot contain NAs")
+        if (length(x) != 0L && min(x) < 0L)
+            stop("when 'x' is an integer vector, ",
+                 "it cannot contain negative values")
         if (!is.null(NG)) {
             ## When 'NG' (number of groups) is supplied, then 'x' is considered
             ## to represent the group assignment of a collection of 'length(x)'
@@ -756,7 +798,7 @@ PartitioningByWidth <- function(x=integer(), NG=NULL, names=NULL)
             ## ADDITIONALLY, 'x' must be *sorted* (not strictly) so it can be
             ## reconstructed from the object returned by PartitioningByWidth()
             ## by doing togroup() on that object.
-            if (isNotSorted(x))
+            if (S4Vectors:::isNotSorted(x))
                 stop("when 'x' is an integer vector, it must be sorted")
             if (!isSingleNumber(NG))
                 stop("'NG' must be either NULL or a single integer")
@@ -798,6 +840,64 @@ setAs("Ranges", "PartitioningByWidth",
     }
 )
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### PartitioningMap contains PartitioningByEnd and one additional slot,
+### 'mapOrder', to specify a different order. This object is used by the 
+### pack() function in GenomicFiles and is put in @partitioning of a 
+### GRangesList of pack()ed ranges. 'mapOrder' records the order of the
+### unpacked() ranges. 
+### 
+
+setClass("PartitioningMap",
+    contains="PartitioningByEnd",
+    representation(
+        mapOrder="integer"
+    ),
+    prototype(
+        mapOrder=integer()
+    )
+)
+
+setGeneric("mapOrder", function(x) standardGeneric("mapOrder"))
+setMethod("mapOrder", "PartitioningMap", function(x) x@mapOrder)
+
+.valid.PartitioningMap <- function(x)
+{
+    if (length(x) == 0L)
+        return(NULL)
+    if (S4Vectors:::anyMissing(mapOrder(x)))
+        return("mapOrder cannot contain NA values")
+    if (any(mapOrder(x) < 0L))
+        return("mapOrder values cannot be negative")
+    if (!is.null(names(mapOrder(x))))
+        return("the mapOrder should not be named")
+
+    if (length(maporder <- mapOrder(x))) {
+        maxorder <- max(maporder)
+        if (max(maporder) > max(end(x)))
+            return("max mapOrder value must be == max(end(object))")
+    }
+    NULL
+}
+
+setValidity2("PartitioningMap", .valid.PartitioningMap)
+
+PartitioningMap <- function(x=integer(), mapOrder=integer(), ...)
+    new("PartitioningMap", PartitioningByEnd(x=x), mapOrder=mapOrder, ...)
+
+setAs("PartitioningByEnd", "PartitioningMap",
+    function(from)
+        new("PartitioningMap", from, mapOrder=numeric())
+)
+
+setMethod("show", "PartitioningMap", 
+    function(object)
+    {
+        cat(class(object), " of length ", length(object), "\n")
+        cat("mapOrder: ", mapOrder(object), "\n")
+        print(PartitioningByEnd(object))
+    }
+) 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### findOverlaps()
@@ -842,7 +942,7 @@ findOverlaps_Ranges_Partitioning <- function(query, subject,
     q_end2subject <- findInterval(q_end, vec)
     q_hits <- rep.int(seq_len(length(q_start)),
                       q_end2subject - q_start2subject + 1L)
-    s_hits <- mseq(q_start2subject, q_end2subject)
+    s_hits <- S4Vectors:::mseq(q_start2subject, q_end2subject)
     ## If 'query' is a Partitioning object, all hits are guaranteed to be
     ## valid.
     if (!is(query, "Partitioning")) {
@@ -858,7 +958,6 @@ findOverlaps_Ranges_Partitioning <- function(query, subject,
         s_hits <- s_idx[s_hits]
 
     ## Make and return Hits object.
-    new2("Hits", queryHits=q_hits, subjectHits=s_hits,
-                 queryLength=q_len, subjectLength=s_len)
+    Hits(q_hits, s_hits, q_len, s_len)
 }   
 
