@@ -46,20 +46,17 @@ setMethod("names", "IRanges", function(x) x@NAMES)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Efficient "isNormal" method for IRanges objects.
+### isEmpty(), isNormal(), isDisjoint()
+###
 
 setMethod("isNormal", "IRanges",
     function(x) .Call2("IRanges_isNormal", x, PACKAGE="IRanges")
 )
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The "isEmpty" default method for Ranges objects would work just fine on a
-### NormalIRanges object but we can take advantage of the normality to make
-### it slightly more efficient.
-###
-
+### Fast methods for NormalIRanges objects.
 setMethod("isEmpty", "NormalIRanges", function(x) length(x) == 0L)
+setMethod("isNormal", "NormalIRanges", function(x) TRUE)
+setMethod("isDisjoint", "NormalIRanges", function(x) TRUE)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -139,7 +136,7 @@ newNormalIRangesFromIRanges <- function(x, check=TRUE)
         stop("'check' must be TRUE or FALSE")
     ## Check only what needs to be checked.
     if (check)
-        stopIfProblems(.valid.NormalIRanges(x))
+        S4Vectors:::stopIfProblems(.valid.NormalIRanges(x))
     ## Make a "hard copy" of the slots. No need to check anything!
     new2("NormalIRanges", start=x@start, width=x@width, NAMES=x@NAMES, check=FALSE)
 }
@@ -188,7 +185,7 @@ setAs("numeric", "NormalIRanges",
 {
     old_start <- start(x)
     ## Use 'x@start[]' instead of just 'x@start' so the right value is recycled
-    x@start[] <- numeric2integer(value)
+    x@start[] <- S4Vectors:::numeric2integer(value)
     x@width <- width(x) - start(x) + old_start
     x
 }
@@ -197,7 +194,7 @@ setAs("numeric", "NormalIRanges",
 `unsafe.end<-` <- function(x, value)
 {
     ## Use 'x@width[]' instead of just 'x@width' so the right value is recycled
-    x@width[] <- width(x) + numeric2integer(value) - end(x)
+    x@width[] <- width(x) + S4Vectors:::numeric2integer(value) - end(x)
     x
 }
 
@@ -205,7 +202,7 @@ setAs("numeric", "NormalIRanges",
 `unsafe.width<-` <- function(x, value)
 {
     ## Use 'x@width[]' instead of just 'x@width' so the right value is recycled
-    x@width[] <- numeric2integer(value)
+    x@width[] <- S4Vectors:::numeric2integer(value)
     x
 }
 
@@ -228,7 +225,7 @@ setAs("numeric", "NormalIRanges",
 unsafe.update <- function(object, ...)
 {
     valid_argnames <- c("start", "end", "width", "names")
-    args <- extraArgsAsList(valid_argnames, ...)
+    args <- S4Vectors:::extraArgsAsList(valid_argnames, ...)
     argnames <- names(args)
     sew <- c("start", "end", "width")
     narg_in_sew <- sum(sew %in% argnames)
@@ -251,8 +248,8 @@ unsafe.update <- function(object, ...)
             start <- args$start
             width <- args$width
         }
-        object@start <- numeric2integer(start)
-        object@width <- numeric2integer(width)
+        object@start <- S4Vectors:::numeric2integer(start)
+        object@width <- S4Vectors:::numeric2integer(width)
         object@NAMES <- args$names
         return(object)
     }
@@ -282,40 +279,46 @@ unsafe.update <- function(object, ...)
 ### replacement methods below need to be overridden for this new class.
 ###
 
+.IRanges.replace_start <- function(x, check=TRUE, value)
+{
+    if (!isTRUEorFALSE(check))
+        stop("'check' must be TRUE or FALSE")
+    unsafe.start(x) <- value
+    if (check)
+        validObject(x)
+    x
+}
+
 setReplaceMethod("start", "IRanges",
-    function(x, check=TRUE, value)
-    {
-        if (!isTRUEorFALSE(check))
-            stop("'check' must be TRUE or FALSE")
-        unsafe.start(x) <- value
-        if (check)
-            validObject(x)
-        x
-    }
+    function(x, ..., value) .IRanges.replace_start(x, ..., value=value)
 )
 
-setReplaceMethod("width", "IRanges",
-    function(x, check=TRUE, value)
-    {
-        if (!isTRUEorFALSE(check))
-            stop("'check' must be TRUE or FALSE")
-        unsafe.width(x) <- value
-        if (check)
-            validObject(x)
-        x
-    }
-)
+.IRanges.replace_end <- function(x, check=TRUE, value)
+{
+    if (!isTRUEorFALSE(check))
+        stop("'check' must be TRUE or FALSE")
+    unsafe.end(x) <- value
+    if (check)
+        validObject(x)
+    x
+}
 
 setReplaceMethod("end", "IRanges",
-    function(x, check=TRUE, value)
-    {
-        if (!isTRUEorFALSE(check))
-            stop("'check' must be TRUE or FALSE")
-        unsafe.end(x) <- value
-        if (check)
-            validObject(x)
-        x
-    }
+    function(x, ..., value) .IRanges.replace_end(x, ..., value=value)
+)
+
+.IRanges.replace_width <- function(x, check=TRUE, value)
+{
+    if (!isTRUEorFALSE(check))
+        stop("'check' must be TRUE or FALSE")
+    unsafe.width(x) <- value
+    if (check)
+        validObject(x)
+    x
+}
+
+setReplaceMethod("width", "IRanges",
+    function(x, ..., value) .IRanges.replace_width(x, ..., value=value)
 )
 
 setReplaceMethod("names", "IRanges",
@@ -365,13 +368,12 @@ setMethod("update", "IRanges",
 setMethod("extractROWS", "IRanges",
     function(x, i)
     {
-        if (missing(i) || !is(i, "Ranges"))
-            i <- normalizeSingleBracketSubscript(i, x)
-        if (is(x, "NormalIRanges")
-         && ((is.integer(i) && isNotStrictlySorted(i))
-          || (is(i, "Ranges") && !isNormal(i))))
-            stop("subscript cannot contain duplicates and must preserve the ",
-                 "order of elements when subsetting a ", class(x), " object")
+        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+        if (is(x, "NormalIRanges")) {
+            if (!isStrictlySorted(i))
+                stop("subscript must extract elements at strictly sorted ",
+                     "positions when\n  subsetting a ", class(x), " object")
+        }
         ans_start <- extractROWS(start(x), i)
         ans_width <- extractROWS(width(x), i)
         ans_names <- extractROWS(names(x), i)
@@ -384,8 +386,7 @@ setMethod("extractROWS", "IRanges",
 setMethod("replaceROWS", "IRanges",
     function(x, i, value)
     {
-        if (missing(i) || !is(i, "Ranges"))
-            i <- normalizeSingleBracketSubscript(i, x)
+        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
         ans_start <- replaceROWS(start(x), i, start(value))
         ans_width <- replaceROWS(width(x), i, width(value))
         ans <- initialize(x, start=ans_start, width=ans_width)
@@ -405,9 +406,9 @@ setMethod("replaceROWS", "IRanges",
 ### It's easy to implement specific "c" methods for IRanges subclasses.
 ### Typically they just need to do something like:
 ###
-###     old_val <- disableValidity()
-###     on.exit(disableValidity(old_val))
-###     disableValidity(TRUE)
+###     old_val <- S4Vectors:::disableValidity()
+###     on.exit(S4Vectors:::disableValidity(old_val))
+###     S4Vectors:::disableValidity(TRUE)
 ###     ans <- callNextMethod(x, ..., recursive=FALSE)
 ###     ...
 ###
@@ -501,7 +502,7 @@ setMethod("c", "IRanges",
         if (ignore.mcols) {
           mcols(ans) <- NULL
         } else  {
-          mcols(ans) <- do.call(rbind.mcols, args)
+          mcols(ans) <- do.call(S4Vectors:::rbind_mcols, args)
         }
         
         validObject(ans)
