@@ -77,61 +77,49 @@ setValidity2("NormalIRangesList", .valid.NormalIRangesList)
 ###
 
 setMethod("start", "RangesList",
-          function(x) newList("SimpleIntegerList", lapply(x, start)))
+          function(x) S4Vectors:::new_SimpleList_from_list("SimpleIntegerList",
+                                                           lapply(x, start)))
 setMethod("end", "RangesList",
-          function(x) newList("SimpleIntegerList", lapply(x, end)))
+          function(x) S4Vectors:::new_SimpleList_from_list("SimpleIntegerList",
+                                                           lapply(x, end)))
 setMethod("width", "RangesList",
-          function(x) newList("SimpleIntegerList", lapply(x, width)))
-setGeneric(".SEW<-", signature="x",  # not exported
-           function(x, FUN, check=TRUE, value) standardGeneric(".SEW<-"))
-setReplaceMethod(".SEW", "RangesList",
-                 function(x, FUN, check=TRUE, value)
-                 {
-                   if (!isTRUEorFALSE(check))
-                     stop("'check' must be TRUE or FALSE")
-                   if (extends(class(value), "IntegerList")) {
-                     if (!identical(lapply(x, names), lapply(value, names)) &&
-                         !all(elementLengths(x) == elementLengths(value)))
-                       stop("'value' must have same length and names as current 'ranges'")
-                   } else if (is.numeric(value)) {
-                     lelts <- sum(elementLengths(x))
-                     if (lelts != length(value))
-                       value <- rep(value, length.out = lelts)
-                     if (!is.integer(value))
-                       value <- as.integer(value)
-                     value <- split(value, factor(space(x), names(x)))
-                   } else {
-                     stop("'value' must extend class IntegerList or integer")
-                   }
-                   FUN <- match.fun(FUN)
-                   for (i in seq_len(length(x)))
-                     x[[i]] <- FUN(x[[i]], check = check, value = value[[i]])
-                   x
-                 })
+          function(x) S4Vectors:::new_SimpleList_from_list("SimpleIntegerList",
+                                                           lapply(x, width)))
+
+setGeneric(".replaceSEW", signature="x",  # not exported
+           function(x, FUN, ..., value) standardGeneric(".replaceSEW"))
+setMethod(".replaceSEW", "RangesList",
+    function(x, FUN, ..., value)
+    {
+        if (extends(class(value), "IntegerList")) {
+            if (!identical(lapply(x, names), lapply(value, names)) &&
+                !all(elementLengths(x) == elementLengths(value)))
+                stop("'value' must have same length and names as current 'ranges'")
+        } else if (is.numeric(value)) {
+            lelts <- sum(elementLengths(x))
+            if (lelts != length(value))
+                value <- rep(value, length.out = lelts)
+            if (!is.integer(value))
+                value <- as.integer(value)
+            value <- split(value, factor(space(x), names(x)))
+        } else {
+            stop("'value' must extend class IntegerList or integer")
+        }
+        FUN <- match.fun(FUN)
+        for (i in seq_len(length(x)))
+            x[[i]] <- FUN(x[[i]], ..., value = value[[i]])
+        x
+    }
+)
 setReplaceMethod("start", "RangesList",
-                 function(x, check=TRUE, value)
-                 {
-                   if (!isTRUEorFALSE(check))
-                     stop("'check' must be TRUE or FALSE")
-                   .SEW(x, FUN = "start<-", check = check) <- value
-                   x
-                 })
+    function(x, ..., value) .replaceSEW(x, "start<-", ..., value=value)
+)
 setReplaceMethod("end", "RangesList",
-                 function(x, check=TRUE, value)
-                 {
-                   if (!isTRUEorFALSE(check))
-                     stop("'check' must be TRUE or FALSE")
-                   .SEW(x, FUN = "end<-", check = check) <- value
-                   x
-                 })
+    function(x, ..., value) .replaceSEW(x, "end<-", ..., value=value)
+)
 setReplaceMethod("width", "RangesList",
-                 function(x, check=TRUE, value)
-                 {
-                   if (!isTRUEorFALSE(check))
-                     stop("'check' must be TRUE or FALSE")
-                   .SEW(x, FUN = "width<-", check = check) <- value
-                   x
-                 })
+    function(x, ..., value) .replaceSEW(x, "width<-", ..., value=value)
+)
 
 setMethod("start", "CompressedIRangesList",
           function(x)
@@ -148,30 +136,30 @@ setMethod("width", "CompressedIRangesList",
           new2("CompressedIntegerList",
                unlistData = width(unlist(x, use.names=FALSE)),
                partitioning = x@partitioning, check=FALSE))
-setReplaceMethod(".SEW", "CompressedIRangesList",
-                 function(x, FUN, check=TRUE, value)
-                 {
-                   if (!isTRUEorFALSE(check))
-                     stop("'check' must be TRUE or FALSE")
-                   if (extends(class(value), "IntegerList")) {
-                     if (!identical(lapply(x, names), lapply(value, names)) &&
-                         !all(elementLengths(x) == elementLengths(value)))
-                       stop("'value' must have same length and names as current 'ranges'")
-                     value <- unlist(value)
-                   } else if (is.numeric(value)) {
-                     lelts <- sum(elementLengths(x))
-                     if (lelts != length(value))
-                       value <- rep(value, length.out = lelts)
-                     if (!is.integer(value))
-                       value <- as.integer(value)
-                   } else {
-                     stop("'value' must extend class IntegerList or integer")
-                   }
-                   FUN <- match.fun(FUN)
-                   slot(x, "unlistData", check=FALSE) <-
-                     FUN(x@unlistData, check = check, value = value)
-                   x
-                 })
+
+setMethod(".replaceSEW", "CompressedIRangesList",
+    function(x, FUN, ..., value)
+    {
+        if (extends(class(value), "IntegerList")) {
+            if (!identical(lapply(x, names), lapply(value, names)) &&
+                !all(elementLengths(x) == elementLengths(value)))
+                stop("'value' must have same length and names as current 'ranges'")
+            value <- unlist(value)
+        } else if (is.numeric(value)) {
+            lelts <- sum(elementLengths(x))
+            if (lelts != length(value))
+                value <- rep(value, length.out = lelts)
+            if (!is.integer(value))
+                value <- as.integer(value)
+        } else {
+            stop("'value' must extend class IntegerList or integer")
+        }
+        FUN <- match.fun(FUN)
+        slot(x, "unlistData", check=FALSE) <-
+                        FUN(x@unlistData, ..., value = value)
+        x
+    }
+)
 
 setGeneric("space", function(x, ...) standardGeneric("space"))
 setMethod("space", "RangesList",
@@ -202,7 +190,11 @@ setReplaceMethod("universe", "RangesList",
                  })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Testing a RangesList object.
+### isNormal(), isDisjoint()
+###
+### Test the list elements of a RangesList object 'x' individually and return
+### a vector of TRUE's or FALSE's parallel to 'x'. More precisely, they're
+### equivalent to 'sapply(x, FUN)', when FUN is 'isNormal' or 'isDisjoint'.
 ###
 
 setMethod("isNormal", "RangesList",
@@ -219,6 +211,10 @@ setMethod("isNormal", "SimpleIRangesList",
 setMethod("whichFirstNotNormal", "RangesList",
           function(x) unlist(lapply(x, whichFirstNotNormal)))
 
+setMethod("isDisjoint", "RangesList",
+          function(x) unlist(lapply(x, isDisjoint)))
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor.
 ###
@@ -232,7 +228,7 @@ RangesList <- function(..., universe = NULL)
     ranges <- ranges[[1L]]
   if (!all(sapply(ranges, is, "Ranges")))
     stop("all elements in '...' must be Ranges objects")
-  ans <- newList("SimpleRangesList", ranges)
+  ans <- S4Vectors:::new_SimpleList_from_list("SimpleRangesList", ranges)
   universe(ans) <- universe
   ans
 }
@@ -271,9 +267,9 @@ IRangesList <- function(..., universe = NULL, compress = TRUE)
     if (!all(sapply(ranges, is, "IRanges")))
       stop("all elements in '...' must be IRanges objects")
     if (compress)
-      ans <- newList("CompressedIRangesList", ranges)
+      ans <- new_CompressedList_from_list("CompressedIRangesList", ranges)
     else
-      ans <- newList("SimpleIRangesList", ranges)
+      ans <- S4Vectors:::new_SimpleList_from_list("SimpleIRangesList", ranges)
   }
   universe(ans) <- universe
   ans
@@ -325,7 +321,11 @@ setMethod("getListElement", "CompressedNormalIRangesList",
                        do.call(c, r[!sapply(r, is.null)])
                      })
     names(ranges) <- names
-    newList(class(x), ranges)
+    if (is(x, "CompressedList"))
+      ans <- new_CompressedList_from_list(class(x), ranges)
+    else
+      ans <- S4Vectors:::new_SimpleList_from_list(class(x), ranges)
+    ans
 }
 
 setMethod("merge", c("RangesList", "missing"),
@@ -393,28 +393,6 @@ setMethod("showAsCell", "RangesList",
 ### Coercion.
 ###
 
-### S3/S4 combo for as.data.frame.RangesList
-as.data.frame.RangesList <- function(x, row.names=NULL, optional=FALSE, ...)
-{
-    if (!(is.null(row.names) || is.character(row.names)))
-        stop("'row.names'  must be NULL or a character vector")
-    x <- as(x, "CompressedIRangesList")
-    spaceLevels <- seq_len(length(x))
-    if (length(names(x)) > 0) {
-        spaceLabels <- names(x)
-    } else {
-        spaceLabels <- as.character(spaceLevels)
-    }
-    data.frame(space =
-               factor(rep.int(seq_len(length(x)), elementLengths(x)),
-                      levels = spaceLevels,
-                      labels = spaceLabels),
-               as.data.frame(unlist(x, use.names = FALSE)),
-               row.names = row.names,
-               stringsAsFactors = FALSE)
-}
-setMethod("as.data.frame", "RangesList", as.data.frame.RangesList)
-
 ### S3/S4 combo for as.list.CompressedNormalIRangesList
 .as.list.CompressedNormalIRangesList <- function(x, use.names=TRUE)
 {
@@ -433,7 +411,8 @@ setMethod("as.list", "CompressedNormalIRangesList",
 setMethod("unlist", "SimpleNormalIRangesList",
           function(x, recursive = TRUE, use.names = TRUE)
           {
-            x <- newList("SimpleIRangesList", lapply(x, as, "IRanges"))
+            x <- S4Vectors:::new_SimpleList_from_list("SimpleIRangesList",
+                                                      lapply(x, as, "IRanges"))
             callGeneric()
           })
 
@@ -450,17 +429,17 @@ setAs("RangesList", "CompressedIRangesList", function(from) {
   if (is(from, "CompressedList"))
     coerceToCompressedList(from, "IRanges")
   ## this case handles RangesList of Partitioning objects (not combinable)
-  else newList("CompressedIRangesList",
-               lapply(from, as, "IRanges"),
-               metadata = metadata(from),
-               mcols = mcols(from))
+  else new_CompressedList_from_list("CompressedIRangesList",
+                                    lapply(from, as, "IRanges"),
+                                    metadata = metadata(from),
+                                    mcols = mcols(from))
 })
 
 .RangesListToSimpleIRangesList <- function(from)
-  newList("SimpleIRangesList",
-          lapply(from, as, "IRanges"),
-          metadata = metadata(from),
-          mcols = mcols(from))
+  S4Vectors:::new_SimpleList_from_list("SimpleIRangesList",
+                                       lapply(from, as, "IRanges"),
+                                       metadata = metadata(from),
+                                       mcols = mcols(from))
 
 ## otherwise, SimpleRangesList->SimpleIRangesList uses a methods package default
 setAs("SimpleRangesList", "SimpleIRangesList", .RangesListToSimpleIRangesList)
@@ -468,10 +447,10 @@ setAs("RangesList", "SimpleIRangesList", .RangesListToSimpleIRangesList)
 
 setAs("RangesList", "SimpleRangesList",
       function(from)
-      newList("SimpleRangesList",
-              lapply(from, as, "Ranges"),
-              metadata = metadata(from),
-              mcols = mcols(from)))
+      S4Vectors:::new_SimpleList_from_list("SimpleRangesList",
+                                           lapply(from, as, "Ranges"),
+                                           metadata = metadata(from),
+                                           mcols = mcols(from)))
 
 setAs("RangesList", "NormalIRangesList",
       function(from)
@@ -484,17 +463,17 @@ setAs("RangesList", "NormalIRangesList",
 
 setAs("RangesList", "CompressedNormalIRangesList",
       function(from)
-      newList("CompressedNormalIRangesList",
-              lapply(from, as, "NormalIRanges"),
-              metadata = metadata(from),
-              mcols = mcols(from)))
+      new_CompressedList_from_list("CompressedNormalIRangesList",
+                                   lapply(from, as, "NormalIRanges"),
+                                   metadata = metadata(from),
+                                   mcols = mcols(from)))
 
 setAs("RangesList", "SimpleNormalIRangesList",
       function(from)
-      newList("SimpleNormalIRangesList",
-              lapply(from, as, "NormalIRanges"),
-              metadata = metadata(from),
-              mcols = mcols(from)))
+      S4Vectors:::new_SimpleList_from_list("SimpleNormalIRangesList",
+                                           lapply(from, as, "NormalIRanges"),
+                                           metadata = metadata(from),
+                                           mcols = mcols(from)))
 
 setAs("CompressedIRangesList", "CompressedNormalIRangesList",
       function(from)
@@ -527,17 +506,17 @@ setAs("LogicalList", "IRangesList",
 
 setAs("LogicalList", "CompressedIRangesList",
       function(from)
-      newList("CompressedIRangesList",
-              lapply(from, as, "IRanges"),
-              metadata = metadata(from),
-              mcols = mcols(from)))
+      new_CompressedList_from_list("CompressedIRangesList",
+                                   lapply(from, as, "IRanges"),
+                                   metadata = metadata(from),
+                                   mcols = mcols(from)))
 
 setAs("LogicalList", "SimpleIRangesList",
       function(from)
-      newList("SimpleIRangesList",
-              lapply(from, as, "IRanges"),
-              metadata = metadata(from),
-              mcols = mcols(from)))
+      S4Vectors:::new_SimpleList_from_list("SimpleIRangesList",
+                                           lapply(from, as, "IRanges"),
+                                           metadata = metadata(from),
+                                           mcols = mcols(from)))
 
 setAs("LogicalList", "NormalIRangesList",
       function(from)
@@ -550,17 +529,17 @@ setAs("LogicalList", "NormalIRangesList",
 
 setAs("LogicalList", "CompressedNormalIRangesList",
       function(from)
-      newList("CompressedNormalIRangesList",
-              lapply(from, as, "NormalIRanges"),
-              metadata = metadata(from),
-              mcols = mcols(from)))
+      new_CompressedList_from_list("CompressedNormalIRangesList",
+                                   lapply(from, as, "NormalIRanges"),
+                                   metadata = metadata(from),
+                                   mcols = mcols(from)))
 
 setAs("LogicalList", "SimpleNormalIRangesList",
       function(from)
-      newList("SimpleNormalIRangesList",
-              lapply(from, as, "NormalIRanges"),
-              metadata = metadata(from),
-              mcols = mcols(from)))
+      S4Vectors:::new_SimpleList_from_list("SimpleNormalIRangesList",
+                                           lapply(from, as, "NormalIRanges"),
+                                           metadata = metadata(from),
+                                           mcols = mcols(from)))
 
 setAs("RleList", "IRangesList",
       function(from)
@@ -576,13 +555,13 @@ setAs("RleList", "CompressedIRangesList",
       {
         if ((length(from) > 0) &&
             (!is.logical(runValue(from[[1L]])) ||
-             anyMissing(runValue(from[[1L]]))))
+             S4Vectors:::anyMissing(runValue(from[[1L]]))))
           stop("cannot coerce a non-logical 'RleList' or a logical 'RleList' ",
                "with NAs to a CompressedIRangesList object")
-        newList("CompressedIRangesList",
-                lapply(from, as, "IRanges"),
-                metadata = metadata(from),
-                mcols = mcols(from))
+        new_CompressedList_from_list("CompressedIRangesList",
+                                     lapply(from, as, "IRanges"),
+                                     metadata = metadata(from),
+                                     mcols = mcols(from))
       })
 
 setAs("CompressedRleList", "CompressedIRangesList",
@@ -590,7 +569,7 @@ setAs("CompressedRleList", "CompressedIRangesList",
       {
         if ((length(from) > 0) &&
             (!is.logical(runValue(from[[1L]])) ||
-             anyMissing(runValue(from[[1L]]))))
+             S4Vectors:::anyMissing(runValue(from[[1L]]))))
           stop("cannot coerce a non-logical 'RleList' or a logical 'RleList' ",
                "with NAs to a CompressedIRangesList object")
         ranges <- as(unlist(from, use.names = FALSE), "IRanges")
@@ -606,13 +585,13 @@ setAs("RleList", "SimpleIRangesList",
       {
         if ((length(from) > 0) &&
             (!is.logical(runValue(from[[1L]])) ||
-             anyMissing(runValue(from[[1L]]))))
+             S4Vectors:::anyMissing(runValue(from[[1L]]))))
           stop("cannot coerce a non-logical 'RleList' or a logical 'RleList' ",
                "with NAs to a SimpleIRangesList object")
-        newList("SimpleIRangesList",
-                lapply(from, as, "IRanges"),
-                metadata = metadata(from),
-                mcols = mcols(from))
+        S4Vectors:::new_SimpleList_from_list("SimpleIRangesList",
+                                             lapply(from, as, "IRanges"),
+                                             metadata = metadata(from),
+                                             mcols = mcols(from))
       })
 
 
@@ -630,13 +609,13 @@ setAs("RleList", "CompressedNormalIRangesList",
       {
         if ((length(from) > 0) &&
             (!is.logical(runValue(from[[1L]])) ||
-             anyMissing(runValue(from[[1L]]))))
+             S4Vectors:::anyMissing(runValue(from[[1L]]))))
           stop("cannot coerce a non-logical 'RleList' or a logical 'RleList' ",
                "with NAs to a CompressedNormalIRangesList object")
-        newList("CompressedNormalIRangesList",
-                lapply(from, as, "NormalIRanges"),
-                metadata = metadata(from),
-                mcols = mcols(from))
+        new_CompressedList_from_list("CompressedNormalIRangesList",
+                                     lapply(from, as, "NormalIRanges"),
+                                     metadata = metadata(from),
+                                     mcols = mcols(from))
       })
 
 setAs("RleList", "SimpleNormalIRangesList",
@@ -644,15 +623,18 @@ setAs("RleList", "SimpleNormalIRangesList",
       {
         if ((length(from) > 0) &&
             (!is.logical(runValue(from[[1L]])) ||
-             anyMissing(runValue(from[[1L]]))))
+             S4Vectors:::anyMissing(runValue(from[[1L]]))))
           stop("cannot coerce a non-logical 'RleList' or a logical 'RleList' ",
                "with NAs to a SimpleNormalIRangesList object")
-        newList("SimpleNormalIRangesList",
-                lapply(from, as, "NormalIRanges"),
-                metadata = metadata(from),
-                mcols = mcols(from))
+        S4Vectors:::new_SimpleList_from_list("SimpleNormalIRangesList",
+                                             lapply(from, as, "NormalIRanges"),
+                                             metadata = metadata(from),
+                                             mcols = mcols(from))
       })
 
+setAs("list", "RangesList", function(from) {
+  S4Vectors:::coerceToSimpleList(from, "Ranges")
+})
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### The "max" and "min" methods for NormalIRangesList objects.
@@ -662,7 +644,7 @@ CompressedNormalIRangesList.max <- function(x, use.names)
 {
     if (!is(x, "CompressedNormalIRangesList"))
         stop("'x' must be a CompressedNormalIRangesList object")
-    use.names <- normargUseNames(use.names)
+    use.names <- S4Vectors:::normargUseNames(use.names)
     .Call2("CompressedNormalIRangesList_max", x, use.names, PACKAGE="IRanges")
 }
 
@@ -677,7 +659,7 @@ CompressedNormalIRangesList.min <- function(x, use.names)
 {
     if (!is(x, "CompressedNormalIRangesList"))
         stop("'x' must be a CompressedNormalIRangesList object")
-    use.names <- normargUseNames(use.names)
+    use.names <- S4Vectors:::normargUseNames(use.names)
     .Call2("CompressedNormalIRangesList_min", x, use.names, PACKAGE="IRanges")
 }
 
