@@ -17,31 +17,16 @@
 ### relistToClass()
 ###
 
-setGeneric("relistToClass",
-    function(x) standardGeneric("relistToClass")
-)
+setGeneric("relistToClass", function(x) standardGeneric("relistToClass"))
 
-setMethod("relistToClass", "ANY",
-    function(x)
-    {
-        selectListClassName(class(x))
-    }
-)
+.selectListClassName <- function(x) {
+  cn <- S4Vectors:::listClassName("Compressed", x)
+  if (cn == "CompressedList")
+    cn <- S4Vectors:::listClassName("Simple", x)
+  cn
+}
 
-### Deprecated!
-setGeneric("splitAsListReturnedClass",
-    function(x) standardGeneric("splitAsListReturnedClass")
-)
-
-setMethod("splitAsListReturnedClass", "ANY",
-    function(x) {
-      .Deprecated("relistToClass")
-      cn <- listClassName("Compressed", class(x))
-      if (cn == "CompressedList")
-        cn <- listClassName("Simple", class(x))
-      cn
-    }
-)
+setMethod("relistToClass", "ANY", function(x) .selectListClassName(class(x)))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -89,7 +74,7 @@ setMethod("relist", c("ANY", "PartitioningByEnd"),
         #                                         start=skeleton_start[i],
         #                                         end=skeleton_end[i]))
 
-        newList(ans_class, listData)
+        S4Vectors:::new_SimpleList_from_list(ans_class, listData)
     }
 )
 
@@ -117,13 +102,17 @@ setMethod("relist", c("Vector", "list"),
 {
     if (length(f) > NROW(x))
         stop("'f' cannot be longer than 'NROW(x)' when it's an integer vector")
-    idx <- orderInteger(f)
-    tmp <- Rle(f[idx])
-    f <- cumsum(runLength(tmp))
-    names(f) <- as.character(runValue(tmp))
     if (!identical(drop, FALSE))
         warning("'drop' is ignored when 'f' is an integer vector")
-    x <- extractROWS(x, idx)
+    f_is_not_sorted <- S4Vectors:::isNotSorted(f)
+    if (f_is_not_sorted) {
+        idx <- S4Vectors:::orderInteger(f)
+        f <- f[idx]
+        x <- extractROWS(x, idx)
+    }
+    tmp <- Rle(f)
+    f <- cumsum(runLength(tmp))
+    names(f) <- as.character(runValue(tmp))
     f <- PartitioningByEnd(f)
     relist(x, f)
 }
@@ -137,13 +126,16 @@ setMethod("relist", c("Vector", "list"),
     f <- as.integer(f)
     if (f_len > x_NROW)
         f <- head(f, n=x_NROW)
-    idx <- orderInteger(f)
+    f_is_not_sorted <- S4Vectors:::isNotSorted(f)
+    if (f_is_not_sorted) {
+        idx <- S4Vectors:::orderInteger(f)
+        x <- extractROWS(x, idx)
+    }
     f <- tabulate(f, nbins=length(f_levels))
     names(f) <- f_levels
     if (drop)
         f <- f[f != 0L]
     f <- cumsum(f)
-    x <- extractROWS(x, idx)
     f <- PartitioningByEnd(f)
     relist(x, f)
 }
@@ -153,16 +145,21 @@ setMethod("relist", c("Vector", "list"),
 {
     if (length(f) > NROW(x))
         stop("'f' cannot be longer than data when it's an integer-Rle")
-    f_vals <- runValue(f)
-    f_lens <- runLength(f)
-    idx <- orderInteger(f_vals)
-    xranges <- successiveIRanges(f_lens)[idx]
-    tmp <- Rle(f_vals[idx], f_lens[idx])
-    f <- cumsum(runLength(tmp))
-    names(f) <- as.character(runValue(tmp))
     if (!identical(drop, FALSE))
         warning("'drop' is ignored when 'f' is an integer-Rle")
-    x <- extractROWS(x, xranges)
+    f_vals <- runValue(f)
+    f_lens <- runLength(f)
+    f_is_not_sorted <- S4Vectors:::isNotSorted(f_vals)
+    if (f_is_not_sorted) {
+        idx <- S4Vectors:::orderInteger(f_vals)
+        xranges <- successiveIRanges(f_lens)[idx]
+        f_vals <- f_vals[idx]
+        f_lens <- f_lens[idx]
+        x <- extractROWS(x, xranges)
+    }
+    tmp <- Rle(f_vals, f_lens)
+    f <- cumsum(runLength(tmp))
+    names(f) <- as.character(runValue(tmp))
     f <- PartitioningByEnd(f)
     relist(x, f)
 }
@@ -187,43 +184,86 @@ setMethod("relist", c("Vector", "list"),
     f_lens <- runLength(f)
     f_levels <- levels(f_vals)
     f_vals <- as.integer(f_vals)
-    idx <- orderInteger(f_vals)
-    xranges <- successiveIRanges(f_lens)[idx]
-    ## Using tabulate2() is 5x faster than doing:
+    f_is_not_sorted <- S4Vectors:::isNotSorted(f_vals)
+    if (f_is_not_sorted) {
+        idx <- S4Vectors:::orderInteger(f_vals)
+        xranges <- successiveIRanges(f_lens)[idx]
+        x <- extractROWS(x, xranges)
+    }
+    ## Using S4Vectors:::tabulate2() is 5x faster than doing:
     ##   f <- integer(length(f_levels))
     ##   tmp <- Rle(f_vals[idx], f_lens[idx])
     ##   f[runValue(tmp)] <- runLength(tmp)
-    f <- tabulate2(f_vals, nbins=length(f_levels), weight=f_lens)
+    f <- S4Vectors:::tabulate2(f_vals, nbins=length(f_levels), weight=f_lens)
     names(f) <- f_levels
     if (drop)
         f <- f[f != 0L]
     f <- cumsum(f)
-    x <- extractROWS(x, xranges)
     f <- PartitioningByEnd(f)
     relist(x, f)
 }
 
-splitAsList <- function(x, f, drop=FALSE)
+setGeneric("splitAsList",
+           function(x, f, drop=FALSE, ...) standardGeneric("splitAsList"),
+           signature = c("x", "f"))
+
+
+### Took this out of the still-in-incubation LazyList package
+factorsToTableIndices <- function(factors) {
+  nI <- length(factors)
+  nx <- length(factors[[1L]])
+  useRle <- any(vapply(factors, is, logical(1), "Rle"))
+  if (useRle) {
+    group <- as(factors[[1L]], "Rle")
+    runValue(group) <- as.integer(runValue(group))
+  } else {
+    group <- as.integer(factors[[1L]])
+  }
+  ngroup <- nlevels(factors[[1L]])
+  for (i in tail(seq_len(nI), -1L)) {
+    index <- factors[[i]]
+    if (useRle) {
+      offset <- as(index, "Rle")
+      runValue(offset) <- ngroup * (as.integer(runValue(offset)) - 1L)
+    } else {
+      offset <- ngroup * (as.integer(index) - 1L)
+    }
+    group <- group + offset
+    ngroup <- ngroup * nlevels(index)
+  }
+  as.vector(group)
+}
+
+normSplitFactor <- function(f, x_NROW) {
+  if (is.list(f) || is(f, "List")) {
+    f <- factorsToTableIndices(f)
+  }
+  f_len <- length(f)
+  if (f_len < x_NROW) {
+    if (f_len == 0L)
+      stop("split factor has length 0 but 'NROW(x)' is > 0")
+    if (x_NROW %% f_len != 0L)
+      warning("'NROW(x)' is not a multiple of split factor length")
+    f <- rep(f, length.out=x_NROW)
+  }
+  f
+}
+
+splitAsList_default <- function(x, f, drop=FALSE)
 {
     if (!isTRUEorFALSE(drop))
         stop("'drop' must be TRUE or FALSE")
-    if (!((is.vector(f) && is.atomic(f)) || is.factor(f) || is(f, "Rle")))
-        stop("'f' must be an atomic vector or a factor (possibly in Rle form)")
     x_NROW <- NROW(x)
-    f_len <- length(f)
-    if (f_len < x_NROW) {
-        if (f_len == 0L)
-            stop("'length(f)' is 0 but 'NROW(x)' is > 0")
-        if (x_NROW %% f_len != 0L)
-            warning("'NROW(x)' is not a multiple of 'length(f)'")
-        f <- rep(f, length.out=x_NROW)
-    }
-    na_idx <- which(is.na(f))
+
+    f <- normSplitFactor(f, x_NROW)
+    is_na <- is.na(f)
+    na_idx <- which(is_na)
     if (length(na_idx) != 0L) {
         keep_idx <- seq_len(x_NROW)[-na_idx]
         x <- extractROWS(x, keep_idx)
         f <- f[keep_idx]
     }
+    
     if (is.integer(f))
         return(.splitAsList_by_integer(x, f, drop))
     if (is.vector(f) && is.atomic(f))
@@ -239,6 +279,13 @@ splitAsList <- function(x, f, drop=FALSE)
     return(.splitAsList_by_Rle(x, f, drop))
 }
 
+setMethod("splitAsList", c("ANY", "vectorORfactor"),
+          function(x, f, drop=FALSE) splitAsList_default(x, f, drop=drop))
+setMethod("splitAsList", c("ANY", "Rle"),
+          function(x, f, drop=FALSE) splitAsList_default(x, f, drop=drop))
+setMethod("splitAsList", c("ANY", "List"),
+          function(x, f, drop=FALSE) splitAsList_default(x, f, drop=drop))
+
 setMethod("split", c("Vector", "ANY"),
     function(x, f, drop=FALSE) splitAsList(x, f, drop=drop)
 )
@@ -290,4 +337,3 @@ setMethod("extractList", c("ANY", "ANY"),
         relist(extractROWS(x, unlisted_i), i)
     }
 )
-
