@@ -92,7 +92,7 @@ setGeneric("reduce", signature="x",
 
 setMethod("reduce", "IRanges",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                with.revmap=FALSE, with.mapping=FALSE,
+                with.revmap=FALSE,
                 with.inframe.attrib=FALSE)
     {
         if (!isTRUEorFALSE(drop.empty.ranges))
@@ -105,22 +105,12 @@ setMethod("reduce", "IRanges",
             stop("'min.gapwidth' must be non-negative")
         if (!isTRUEorFALSE(with.revmap))
             stop("'with.revmap' must be TRUE or FALSE")
-        if (!identical(with.mapping, FALSE)) {
-            msg <- c("  The 'with.mapping' argument is deprecated.\n",
-                     "  Please use the 'with.revmap' argument instead.")
-            .Deprecated(msg=msg)
-            if (!isTRUEorFALSE(with.mapping))
-                stop("'with.mapping' must be TRUE or FALSE")
-            if (with.revmap && with.mapping)
-                stop("'with.revmap' and 'with.mapping' cannot be ",
-                     "both set to TRUE")
-        }
         if (!isTRUEorFALSE(with.inframe.attrib))
             stop("'with.inframe.attrib' must be TRUE or FALSE")
         C_ans <- .Call2("Ranges_reduce",
                         start(x), width(x),
                         drop.empty.ranges, min.gapwidth,
-                        with.revmap || with.mapping,
+                        with.revmap,
                         with.inframe.attrib,
                         PACKAGE="IRanges")
         ans <- unsafe.update(x, start=C_ans$start, width=C_ans$width,
@@ -129,10 +119,6 @@ setMethod("reduce", "IRanges",
             revmap <- IntegerList(C_ans$revmap)
             mcols(ans) <- DataFrame(revmap=revmap)
         }
-        if (with.mapping) {
-            mapping <- IntegerList(C_ans$revmap)
-            mcols(ans) <- DataFrame(mapping=mapping)
-        }
         if (with.inframe.attrib) {
             inframe <- new2("IRanges", start=C_ans$inframe.start,
                                        width=width(x), check=FALSE)
@@ -144,13 +130,13 @@ setMethod("reduce", "IRanges",
 
 setMethod("reduce", "Ranges",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                with.revmap=FALSE, with.mapping=FALSE,
+                with.revmap=FALSE, 
                 with.inframe.attrib=FALSE)
     {
         ir <- as(x, "IRanges")
         y <- reduce(ir, drop.empty.ranges=drop.empty.ranges,
                         min.gapwidth=min.gapwidth,
-                        with.revmap=with.revmap, with.mapping=with.mapping,
+                        with.revmap=with.revmap,
                         with.inframe.attrib=with.inframe.attrib)
         as(y, class(x))
     }
@@ -158,13 +144,13 @@ setMethod("reduce", "Ranges",
 
 setMethod("reduce", "Views",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                with.revmap=FALSE, with.mapping=FALSE,
+                with.revmap=FALSE,
                 with.inframe.attrib=FALSE)
     {
         x@ranges <- reduce(ranges(x),
                            drop.empty.ranges=drop.empty.ranges,
                            min.gapwidth=min.gapwidth,
-                           with.revmap=with.revmap, with.mapping=with.mapping,
+                           with.revmap=with.revmap,
                            with.inframe.attrib=with.inframe.attrib)
         x
     }
@@ -172,18 +158,18 @@ setMethod("reduce", "Views",
 
 setMethod("reduce", "RangesList",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                with.revmap=FALSE, with.mapping=FALSE,
+                with.revmap=FALSE,
                 with.inframe.attrib=FALSE)
         endoapply(x, reduce, drop.empty.ranges = drop.empty.ranges,
                      min.gapwidth = min.gapwidth,
-                     with.revmap=with.revmap, with.mapping=FALSE,
+                     with.revmap=with.revmap,
                      with.inframe.attrib = with.inframe.attrib))
 
 ### 'with.inframe.attrib' is ignored for now.
 ### TODO: Support 'with.inframe.attrib=TRUE'.
 setMethod("reduce", "CompressedIRangesList",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                with.revmap=FALSE, with.mapping=FALSE,
+                with.revmap=FALSE,
                 with.inframe.attrib=FALSE)
     {
         if (!isTRUEorFALSE(drop.empty.ranges))
@@ -196,22 +182,12 @@ setMethod("reduce", "CompressedIRangesList",
             stop("'min.gapwidth' must be non-negative")
         if (!isTRUEorFALSE(with.revmap))
             stop("'with.revmap' must be TRUE or FALSE")
-        if (!identical(with.mapping, FALSE)) {
-            msg <- c("  The 'with.mapping' argument is deprecated.\n",
-                     "  Please use the 'with.revmap' argument instead.")
-            .Deprecated(msg=msg)
-            if (!isTRUEorFALSE(with.mapping))
-                stop("'with.mapping' must be TRUE or FALSE")
-            if (with.revmap && with.mapping)
-                stop("'with.revmap' and 'with.mapping' cannot be ",
-                     "both set to TRUE")
-        }
         if (!identical(with.inframe.attrib, FALSE))
             stop("'with.inframe.attrib' argument not yet supported ",
                  "when reducing a CompressedIRangesList object")
         C_ans <- .Call2("CompressedIRangesList_reduce",
                         x, drop.empty.ranges, min.gapwidth,
-                        with.revmap || with.mapping,
+                        with.revmap,
                         PACKAGE="IRanges")
         ans_unlistData <- new2("IRanges", start=C_ans$start, width=C_ans$width,
                                           check=FALSE)
@@ -219,10 +195,6 @@ setMethod("reduce", "CompressedIRangesList",
             revmap <- IntegerList(C_ans$revmap)
             mcols(ans_unlistData) <- DataFrame(revmap=revmap)
         }
-        if (with.mapping) {
-            mapping <- IntegerList(C_ans$revmap)
-            mcols(ans_unlistData) <- DataFrame(mapping=mapping)
-        }
         ans_partitioning <- PartitioningByEnd(C_ans$partitioning_by_end)
         names(ans_partitioning) <- names(x)
         relist(ans_unlistData, ans_partitioning)
@@ -312,8 +284,8 @@ setGeneric("gaps", signature="x",
 setMethod("gaps", "IRanges",
     function(x, start=NA, end=NA)
     {
-        start <- normargSingleStartOrNA(start)
-        end <- normargSingleEndOrNA(end)
+        start <- S4Vectors:::normargSingleStartOrNA(start)
+        end <- S4Vectors:::normargSingleEndOrNA(end)
         C_ans <- .Call2("IRanges_gaps",
                         start(x), width(x), start, end,
                         PACKAGE="IRanges")
@@ -355,16 +327,16 @@ setMethod("gaps", "RangesList",
           function(x, start = NA, end = NA)
           {
               lx <- length(x)
-              if (!isNumericOrNAs(start))
+              if (!S4Vectors:::isNumericOrNAs(start))
                   stop("'start' must be an integer vector or NA")
               if (!is.integer(start))
                   start <- as.integer(start)
-              if (!isNumericOrNAs(end))
+              if (!S4Vectors:::isNumericOrNAs(end))
                   stop("'end' must be an integer vector or NA")
               if (!is.integer(end))
                   end <- as.integer(end)
-              start <- IntegerList(as.list(recycleVector(start, lx)))
-              end <- IntegerList(as.list(recycleVector(end, lx)))
+              start <- IntegerList(as.list(S4Vectors:::recycleVector(start, lx)))
+              end <- IntegerList(as.list(S4Vectors:::recycleVector(end, lx)))
               mendoapply(gaps, x, start = start, end = end)
           })
 
@@ -372,17 +344,17 @@ setMethod("gaps", "CompressedIRangesList",
           function(x, start = NA, end = NA)
           {
               lx <- length(x)
-              if (!isNumericOrNAs(start))
+              if (!S4Vectors:::isNumericOrNAs(start))
                   stop("'start' must be an integer vector or NA")
               if (!is.integer(start))
                   start <- as.integer(start)
-              if (!isNumericOrNAs(end))
+              if (!S4Vectors:::isNumericOrNAs(end))
                   stop("'end' must be an integer vector or NA")
               if (!is.integer(end))
                   end <- as.integer(end)
               if ((length(start) != 1) || (length(end) != 1)) {
-                  start <- recycleVector(start, lx)
-                  end <- recycleVector(end, lx)
+                  start <- S4Vectors:::recycleVector(start, lx)
+                  end <- S4Vectors:::recycleVector(end, lx)
               }
               .Call2("CompressedIRangesList_gaps", x, start, end,
                     PACKAGE="IRanges")
@@ -472,30 +444,6 @@ setMethod("disjoin", "IntervalForest",
           function(x, ...)
             as(disjoin(as(x, "CompressedIRangesList"), ...), "IntervalForest"))
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### isDisjoint()
-###
-
-setGeneric("isDisjoint", function(x, ...) standardGeneric("isDisjoint"))
-
-setMethod("isDisjoint", "Ranges",
-    function(x)
-    {
-        x_len <- length(x)
-        if (x_len < 2L)
-            return(TRUE)
-        x_start <- start(x)
-        x_end <- end(x)
-        oo <- order(x)
-        start2 <- x_start[oo]
-        end2 <- x_end[oo]
-        all(start2[-1L] > end2[-x_len])
-    }
-)
-
-setMethod("isDisjoint", "RangesList",
-          function(x) unlist(lapply(x, isDisjoint)))
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### disjointBins()
@@ -508,7 +456,7 @@ setMethod("disjointBins", "Ranges",
     function(x)
     {
         x_ord <- NULL
-        if (isNotSorted(start(x))) { # minimize work for sorted ranges (common)
+        if (S4Vectors:::isNotSorted(start(x))) { # minimize work for sorted ranges (common)
             x_ord <- order(x)
             x <- x[x_ord]
         }
@@ -524,5 +472,6 @@ setMethod("disjointBins", "Ranges",
     }
 )
 
-setMethod("disjointBins", "RangesList", function(x) seqapply(x, disjointBins))
+setMethod("disjointBins", "RangesList",
+          function(x) as(lapply(x, disjointBins), "IntegerList"))
 
