@@ -9,6 +9,8 @@
  *                                                                          *
  ****************************************************************************/
 #include "IRanges.h"
+#include "S4Vectors_interface.h"
+
 #include <stdlib.h> /* for qsort() */
 #include <R_ext/Utils.h> /* for R_CheckUserInterrupt() */
 
@@ -123,7 +125,7 @@ static void compute_int_coverage_in_bufs(const int *SEids, int SEids_len,
 
 	*(values_buf++) = curr_val = 0;
 	curr_pos = 1;
-	_reset_ovflow_flag(); /* we use _safe_int_add() in loop below */
+	reset_ovflow_flag(); /* we use safe_int_add() in loop below */
 	for (i = 0; i < SEids_len; i++, SEids++) {
 		if (i % 500000 == 499999)
 			R_CheckUserInterrupt();
@@ -135,11 +137,11 @@ static void compute_int_coverage_in_bufs(const int *SEids, int SEids_len,
 			curr_weight = - curr_weight;
 			curr_pos += x_width[index];
 		}
-		curr_val = _safe_int_add(curr_val, curr_weight);
+		curr_val = safe_int_add(curr_val, curr_weight);
 		*(values_buf++) = curr_val;
 		*(lengths_buf++) = curr_pos - prev_pos;
 	}
-	if (_get_ovflow_flag())
+	if (get_ovflow_flag())
 		warning("NAs produced by integer overflow");
 	*lengths_buf = cvg_len + 1 - curr_pos;
 	return;
@@ -186,7 +188,7 @@ static SEXP int_coverage_sort(const int *x_start, const int *x_width,
 	if (SEids_len == 0) {
 		//return an Rle with one run of 0's
 		zero = 0;
-		return _integer_Rle_constructor(&zero, 1, &cvg_len, 0);
+		return integer_Rle_constructor(&zero, 1, &cvg_len, 0);
 	}
 	sort_SEids(SEids, SEids_len, x_start, x_width);
 	buf_len = SEids_len + 1;
@@ -195,7 +197,7 @@ static SEXP int_coverage_sort(const int *x_start, const int *x_width,
 	compute_int_coverage_in_bufs(SEids, SEids_len,
 			x_start, x_width, weight, weight_len, cvg_len,
 			values_buf, lengths_buf);
-	return _integer_Rle_constructor(values_buf, buf_len, lengths_buf, 0);
+	return integer_Rle_constructor(values_buf, buf_len, lengths_buf, 0);
 }
 
 static SEXP double_coverage_sort(const int *x_start, const int *x_width,
@@ -211,7 +213,7 @@ static SEXP double_coverage_sort(const int *x_start, const int *x_width,
 	if (SEids_len == 0) {
 		//return an Rle with one run of 0's
 		zero = 0.0;
-		return _numeric_Rle_constructor(&zero, 1, &cvg_len, 0);
+		return numeric_Rle_constructor(&zero, 1, &cvg_len, 0);
 	}
 	sort_SEids(SEids, SEids_len, x_start, x_width);
 	buf_len = SEids_len + 1;
@@ -220,7 +222,7 @@ static SEXP double_coverage_sort(const int *x_start, const int *x_width,
 	compute_double_coverage_in_bufs(SEids, SEids_len,
 			x_start, x_width, weight, weight_len, cvg_len,
 			values_buf, lengths_buf);
-	return _numeric_Rle_constructor(values_buf, buf_len, lengths_buf, 0);
+	return numeric_Rle_constructor(values_buf, buf_len, lengths_buf, 0);
 }
 
 static SEXP coverage_sort(const int *x_start, const int *x_width, int x_len,
@@ -251,7 +253,7 @@ static SEXP int_coverage_hash(
 
 	cvg_buf = (int *) R_alloc((long) cvg_len + 1, sizeof(int));
 	memset(cvg_buf, 0, cvg_len * sizeof(int));
-	_reset_ovflow_flag(); /* we use _safe_int_add() in loop below */
+	reset_ovflow_flag(); /* we use safe_int_add() in loop below */
 	for (i = j = 0; i < x_len; i++, j++, x_start++, x_width++) {
 		if (i % 500000 == 499999)
 			R_CheckUserInterrupt();
@@ -259,19 +261,19 @@ static SEXP int_coverage_hash(
 			j = 0; /* recycle j */
 		cvg_p = cvg_buf + *x_start - 1;
 		w = weight[j];
-		*cvg_p = _safe_int_add(*cvg_p, w);
+		*cvg_p = safe_int_add(*cvg_p, w);
 		cvg_p += *x_width;
-		*cvg_p = _safe_int_add(*cvg_p, - w);
+		*cvg_p = safe_int_add(*cvg_p, - w);
 	}
 	check_recycling_was_round(j, weight_len, weight_label, x_label);
 	cumsum = 0;
 	for (i = 0, cvg_p = cvg_buf; i < cvg_len; i++, cvg_p++) {
-		cumsum = _safe_int_add(*cvg_p, cumsum);
+		cumsum = safe_int_add(*cvg_p, cumsum);
 		*cvg_p = cumsum;
 	}
-	if (_get_ovflow_flag())
+	if (get_ovflow_flag())
 		warning("NAs produced by integer overflow");
-	return _integer_Rle_constructor(cvg_buf, cvg_len, NULL, 0);
+	return integer_Rle_constructor(cvg_buf, cvg_len, NULL, 0);
 }
 
 static SEXP double_coverage_hash(
@@ -302,7 +304,7 @@ static SEXP double_coverage_hash(
 		cumsum += *cvg_p;
 		*cvg_p = cumsum;
 	}
-	return _numeric_Rle_constructor(cvg_buf, cvg_len, NULL, 0);
+	return numeric_Rle_constructor(cvg_buf, cvg_len, NULL, 0);
 }
 
 static SEXP coverage_hash(const int *x_start, const int *x_width, int x_len,
@@ -411,7 +413,7 @@ static int double2int(double x)
  */
 static int shift_and_clip_ranges(const IRanges_holder *x_holder,
 		SEXP shift, int width, int circle_len,
-		RangeAE *out_ranges, int *out_ranges_are_tiles)
+		IntPairAE *out_ranges, int *out_ranges_are_tiles)
 {
 	int x_len, shift_len, cvg_len, auto_cvg_len, prev_end,
 	    i, j, x_start, x_end, shift_elt, tmp;
@@ -448,7 +450,7 @@ static int shift_and_clip_ranges(const IRanges_holder *x_holder,
 		return cvg_len;
 	}
 
-	_RangeAE_set_nelt(out_ranges, 0);
+	IntPairAE_set_nelt(out_ranges, 0);
 	prev_end = 0;
 	for (i = j = 0; i < x_len; i++, j++) {
 		if (j >= shift_len)
@@ -494,8 +496,8 @@ static int shift_and_clip_ranges(const IRanges_holder *x_holder,
 			else
 				*out_ranges_are_tiles = 0;
 		}
-		_RangeAE_insert_at(out_ranges, i,
-				   x_start, x_end - x_start + 1);
+		IntPairAE_insert_at(out_ranges, i,
+				    x_start, x_end - x_start + 1);
 	}
 	check_recycling_was_round(j, shift_len, shift_label, x_label);
 	if (*out_ranges_are_tiles && x_end != cvg_len)
@@ -519,7 +521,7 @@ static int shift_and_clip_ranges(const IRanges_holder *x_holder,
 static SEXP compute_coverage_from_IRanges_holder(
 		const IRanges_holder *x_holder,
 		SEXP shift, int width, SEXP weight, int circle_len,
-		SEXP method, RangeAE *ranges_buf)
+		SEXP method, IntPairAE *ranges_buf)
 {
 	int x_len, cvg_len, out_ranges_are_tiles, weight_len,
 	    effective_method, take_short_path;
@@ -529,8 +531,8 @@ static SEXP compute_coverage_from_IRanges_holder(
 	x_len = _get_length_from_IRanges_holder(x_holder);
 	cvg_len = shift_and_clip_ranges(x_holder, shift, width, circle_len,
 					ranges_buf, &out_ranges_are_tiles);
-	x_start = ranges_buf->start.elts;
-	x_width = ranges_buf->width.elts;
+	x_start = ranges_buf->a.elts;
+	x_width = ranges_buf->b.elts;
 
 	/* Check 'weight'. */
 	check_arg_is_numeric(weight, weight_label);
@@ -576,10 +578,10 @@ static SEXP compute_coverage_from_IRanges_holder(
 			/* Short path for the tiling case. */
 			//Rprintf("taking short path\n");
 			return IS_INTEGER(weight) ?
-			       _integer_Rle_constructor(INTEGER(weight), x_len,
-							x_width, 0) :
-			       _numeric_Rle_constructor(REAL(weight), x_len,
-							x_width, 0);
+			       integer_Rle_constructor(INTEGER(weight), x_len,
+						       x_width, 0) :
+			       numeric_Rle_constructor(REAL(weight), x_len,
+						       x_width, 0);
 		}
 	}
 	//Rprintf("taking normal path\n");
@@ -605,7 +607,7 @@ SEXP IRanges_coverage(SEXP x, SEXP shift, SEXP width, SEXP weight,
 {
 	IRanges_holder x_holder;
 	int x_len;
-	RangeAE ranges_buf;
+	IntPairAE ranges_buf;
 
 	x_holder = _hold_IRanges(x);
 	x_len = _get_length_from_IRanges_holder(&x_holder);
@@ -620,7 +622,7 @@ SEXP IRanges_coverage(SEXP x, SEXP shift, SEXP width, SEXP weight,
 	if (LENGTH(circle_len) != 1)
 		error("'%s' must be a single integer", "circle.length");
 
-	ranges_buf = _new_RangeAE(x_len, 0);
+	ranges_buf = new_IntPairAE(x_len, 0);
 	x_label = "x";
 	shift_label = "shift";
 	width_label = "width";
@@ -657,7 +659,7 @@ SEXP CompressedIRangesList_coverage(SEXP x,
 	CompressedIRangesList_holder x_holder;
 	int x_len, shift_len, width_len, weight_len, circle_lens_len,
 	    i, j, k, l, m;
-	RangeAE ranges_buf;
+	IntPairAE ranges_buf;
 	SEXP ans, ans_elt, shift_elt, weight_elt;
 	IRanges_holder x_elt_holder;
 	char x_label_buf[40], shift_label_buf[40],
@@ -686,7 +688,7 @@ SEXP CompressedIRangesList_coverage(SEXP x,
 	circle_lens_len = LENGTH(circle_lens);
 	check_arg_is_recyclable(circle_lens_len, x_len, "circle.length", "x");
 
-	ranges_buf = _new_RangeAE(0, 0);
+	ranges_buf = new_IntPairAE(0, 0);
 	x_label = x_label_buf;
 	shift_label = shift_label_buf;
 	width_label = width_label_buf;
