@@ -3,6 +3,8 @@
  *                           Author: Herve Pages                            *
  ****************************************************************************/
 #include "IRanges.h"
+#include "S4Vectors_interface.h"
+
 #include <limits.h>
 
 #define R_INT_MIN	(1+INT_MIN)
@@ -42,7 +44,7 @@ static int get_elt_from_CompressedIRangesList_holderlens_max(
 	return ir_len_max;
 }
 
-static int append_IRanges_holder_to_RangeAE(RangeAE *range_ae,
+static int append_IRanges_holder_to_IntPairAE(IntPairAE *intpair_ae,
 		const IRanges_holder *ir_holder)
 {
 	int ir_len, j, start, width;
@@ -51,8 +53,8 @@ static int append_IRanges_holder_to_RangeAE(RangeAE *range_ae,
 	for (j = 0; j < ir_len; j++) {
 		start = _get_start_elt_from_IRanges_holder(ir_holder, j);
 		width = _get_width_elt_from_IRanges_holder(ir_holder, j);
-		_RangeAE_insert_at(range_ae, _RangeAE_get_nelt(range_ae),
-				start, width);
+		IntPairAE_insert_at(intpair_ae, IntPairAE_get_nelt(intpair_ae),
+				    start, width);
 	}
 	return ir_len;
 }
@@ -107,7 +109,7 @@ SEXP IRanges_range(SEXP x)
    Returns the number of ranges that were appended. */
 static int reduce_ranges(const int *x_start, const int *x_width, int x_len,
 		int drop_empty_ranges, int min_gapwidth,
-		int *order_buf, RangeAE *out_ranges,
+		int *order_buf, IntPairAE *out_ranges,
 		IntAEAE *revmap, int *out_inframe_start)
 {
 	int out_len, out_len0, i, j, start_j, width_j, end_j,
@@ -117,8 +119,8 @@ static int reduce_ranges(const int *x_start, const int *x_width, int x_len,
 	if (min_gapwidth < 0)
 		error("IRanges internal error in reduce_ranges(): "
 		      "negative min_gapwidth not supported");
-	_get_order_of_int_pairs(x_start, x_width, x_len, 0, order_buf, 0);
-	out_len = out_len0 = _RangeAE_get_nelt(out_ranges);
+	get_order_of_int_pairs(x_start, x_width, x_len, 0, order_buf, 0);
+	out_len = out_len0 = IntPairAE_get_nelt(out_ranges);
 	for (i = 0; i < x_len; i++) {
 		j = order_buf[i];
 		start_j = x_start[j];
@@ -144,18 +146,18 @@ static int reduce_ranges(const int *x_start, const int *x_width, int x_len,
 			if (width_j != 0
 			 || (!drop_empty_ranges
 			     && (out_len == out_len0
-			         || start_j != out_ranges->start.elts[
+			         || start_j != out_ranges->a.elts[
 					out_len - 1]))) {
 				/* Append to 'out_ranges'. */
-				_RangeAE_insert_at(out_ranges,
+				IntPairAE_insert_at(out_ranges,
 					out_len,
 					start_j, width_j);
 				if (revmap != NULL) {
 					/* Append to 'revmap'. */
-					tmp = _new_IntAE(1, 1, j + 1);
-					_IntAEAE_insert_at(revmap,
-							   out_len,
-							   &tmp);
+					tmp = new_IntAE(1, 1, j + 1);
+					IntAEAE_insert_at(revmap,
+							  out_len,
+							  &tmp);
 					revmap_elt = revmap->elts + out_len;
 				}
 				out_len++;
@@ -168,15 +170,15 @@ static int reduce_ranges(const int *x_start, const int *x_width, int x_len,
 			width_inc = end_j - max_end;
 			if (width_inc > 0) {
 				/* Merge with last range in 'out_ranges'. */
-				out_ranges->width.elts[out_len - 1] += width_inc;
+				out_ranges->b.elts[out_len - 1] += width_inc;
 				max_end = end_j;
 			}
 			if (!(width_j == 0 && drop_empty_ranges)
 			 && revmap != NULL) {
 				/* Append to 'revmap'. */
-				_IntAE_insert_at(revmap_elt,
-						 _IntAE_get_nelt(revmap_elt),
-						 j + 1);
+				IntAE_insert_at(revmap_elt,
+						IntAE_get_nelt(revmap_elt),
+						j + 1);
 			}
 		}
 		if (out_inframe_start != NULL) 
@@ -192,15 +194,15 @@ SEXP Ranges_reduce(SEXP x_start, SEXP x_width, SEXP drop_empty_ranges,
 	int x_len, *inframe_start;
 	const int *x_start_p, *x_width_p;
 	SEXP ans, ans_names, ans_revmap, ans_inframe_start;
-	RangeAE out_ranges;
+	IntPairAE out_ranges;
 	IntAE order_buf;
 	IntAEAE tmp, *revmap;
 
-	x_len = _check_integer_pairs(x_start, x_width,
-				     &x_start_p, &x_width_p,
-				     "start(x)", "width(x)");
+	x_len = check_integer_pairs(x_start, x_width,
+				    &x_start_p, &x_width_p,
+				    "start(x)", "width(x)");
 	if (LOGICAL(with_revmap)[0]) {
-		tmp = _new_IntAEAE(0, 0);
+		tmp = new_IntAEAE(0, 0);
 		revmap = &tmp;
 	} else {
 		revmap = NULL;
@@ -211,8 +213,8 @@ SEXP Ranges_reduce(SEXP x_start, SEXP x_width, SEXP drop_empty_ranges,
 	} else {
 		inframe_start = NULL;
 	}
-	out_ranges = _new_RangeAE(0, 0);
-	order_buf = _new_IntAE(x_len, 0, 0);
+	out_ranges = new_IntPairAE(0, 0);
+	order_buf = new_IntAE(x_len, 0, 0);
 	reduce_ranges(x_start_p, x_width_p, x_len,
 		LOGICAL(drop_empty_ranges)[0], INTEGER(min_gapwidth)[0],
 		order_buf.elts, &out_ranges, revmap, inframe_start);
@@ -226,10 +228,10 @@ SEXP Ranges_reduce(SEXP x_start, SEXP x_width, SEXP drop_empty_ranges,
 	SET_STRING_ELT(ans_names, 3, mkChar("inframe.start"));
 	SET_NAMES(ans, ans_names);
 	UNPROTECT(1);
-	SET_VECTOR_ELT(ans, 0, _new_INTEGER_from_IntAE(&(out_ranges.start)));
-	SET_VECTOR_ELT(ans, 1, _new_INTEGER_from_IntAE(&(out_ranges.width)));
+	SET_VECTOR_ELT(ans, 0, new_INTEGER_from_IntAE(&(out_ranges.a)));
+	SET_VECTOR_ELT(ans, 1, new_INTEGER_from_IntAE(&(out_ranges.b)));
 	if (revmap != NULL) {
-		PROTECT(ans_revmap = _new_LIST_from_IntAEAE(revmap, 0));
+		PROTECT(ans_revmap = new_LIST_from_IntAEAE(revmap, 0));
 		SET_VECTOR_ELT(ans, 2, ans_revmap);
 		UNPROTECT(1);
 	}
@@ -251,31 +253,31 @@ SEXP CompressedIRangesList_reduce(SEXP x, SEXP drop_empty_ranges,
 	IRanges_holder ir_holder;
 	int x_len, in_len_max, i;
 	IntAE order_buf;
-	RangeAE in_ranges, out_ranges;
+	IntPairAE in_ranges, out_ranges;
 	IntAEAE tmp, *revmap;
 
 	x_holder = _hold_CompressedIRangesList(x);
 	x_len = _get_length_from_CompressedIRangesList_holder(&x_holder);
 	if (LOGICAL(with_revmap)[0]) {
-		tmp = _new_IntAEAE(0, 0);
+		tmp = new_IntAEAE(0, 0);
 		revmap = &tmp;
 	} else {
 		revmap = NULL;
 	}
 	in_len_max = get_elt_from_CompressedIRangesList_holderlens_max(&x_holder);
-	order_buf = _new_IntAE(in_len_max, 0, 0);
-	in_ranges = _new_RangeAE(0, 0);
-	out_ranges = _new_RangeAE(0, 0);
+	order_buf = new_IntAE(in_len_max, 0, 0);
+	in_ranges = new_IntPairAE(0, 0);
+	out_ranges = new_IntPairAE(0, 0);
 	PROTECT(ans_partitioning_end = NEW_INTEGER(x_len));
 	for (i = 0; i < x_len; i++) {
 		ir_holder = _get_elt_from_CompressedIRangesList_holder(&x_holder, i);
-		_RangeAE_set_nelt(&in_ranges, 0);
-		append_IRanges_holder_to_RangeAE(&in_ranges, &ir_holder);
-		reduce_ranges(in_ranges.start.elts, in_ranges.width.elts,
-			_RangeAE_get_nelt(&in_ranges),
+		IntPairAE_set_nelt(&in_ranges, 0);
+		append_IRanges_holder_to_IntPairAE(&in_ranges, &ir_holder);
+		reduce_ranges(in_ranges.a.elts, in_ranges.b.elts,
+			IntPairAE_get_nelt(&in_ranges),
 			LOGICAL(drop_empty_ranges)[0], INTEGER(min_gapwidth)[0],
 			order_buf.elts, &out_ranges, revmap, NULL);
-		INTEGER(ans_partitioning_end)[i] = _RangeAE_get_nelt(&out_ranges);
+		INTEGER(ans_partitioning_end)[i] = IntPairAE_get_nelt(&out_ranges);
 	}
 
 	/* Make 'ans' */
@@ -287,23 +289,23 @@ SEXP CompressedIRangesList_reduce(SEXP x, SEXP drop_empty_ranges,
 	SET_STRING_ELT(ans_names, 3, mkChar("partitioning_by_end"));
 	SET_NAMES(ans, ans_names);
 	UNPROTECT(1);
-	SET_VECTOR_ELT(ans, 0, _new_INTEGER_from_IntAE(&(out_ranges.start)));
-	SET_VECTOR_ELT(ans, 1, _new_INTEGER_from_IntAE(&(out_ranges.width)));
+	SET_VECTOR_ELT(ans, 0, new_INTEGER_from_IntAE(&(out_ranges.a)));
+	SET_VECTOR_ELT(ans, 1, new_INTEGER_from_IntAE(&(out_ranges.b)));
 	if (revmap != NULL) {
-		PROTECT(ans_revmap = _new_LIST_from_IntAEAE(revmap, 0));
+		PROTECT(ans_revmap = new_LIST_from_IntAEAE(revmap, 0));
 		SET_VECTOR_ELT(ans, 2, ans_revmap);
 		UNPROTECT(1);
 	}
 	SET_VECTOR_ELT(ans, 3, ans_partitioning_end);
 	UNPROTECT(2);
 /*
-	PROTECT(ans_unlistData = _new_IRanges_from_RangeAE("IRanges",
+	PROTECT(ans_unlistData = _new_IRanges_from_IntPairAE("IRanges",
 			&out_ranges));
 	PROTECT(ans_names = duplicate(_get_CompressedList_names(x)));
 	PROTECT(ans_partitioning = _new_PartitioningByEnd(
 			"PartitioningByEnd",
 			ans_partitioning_end, ans_names));
-	PROTECT(ans = _new_CompressedList(_get_classname(x),
+	PROTECT(ans = _new_CompressedList(get_classname(x),
 			ans_unlistData, ans_partitioning));
 	UNPROTECT(5);
 */
@@ -319,7 +321,7 @@ SEXP CompressedIRangesList_reduce(SEXP x, SEXP drop_empty_ranges,
    Returns the number of ranges that were appended. */
 static int gaps_ranges(const int *x_start, const int *x_width, int x_len,
 		int restrict_start, int restrict_end,
-		int *order_buf, RangeAE *out_ranges)
+		int *order_buf, IntPairAE *out_ranges)
 {
 	int out_len, out_len0, i, j, start_j, width_j, end_j,
 	    max_end, gapstart, gapwidth;
@@ -328,8 +330,8 @@ static int gaps_ranges(const int *x_start, const int *x_width, int x_len,
 		max_end = restrict_start - 1;
 	else
 		max_end = NA_INTEGER;
-	_get_order_of_int_pairs(x_start, x_width, x_len, 0, order_buf, 0);
-	out_len = out_len0 = _RangeAE_get_nelt(out_ranges);
+	get_order_of_int_pairs(x_start, x_width, x_len, 0, order_buf, 0);
+	out_len = out_len0 = IntPairAE_get_nelt(out_ranges);
 	for (i = 0; i < x_len; i++) {
 		j = order_buf[i];
 		width_j = x_width[j];
@@ -347,7 +349,7 @@ static int gaps_ranges(const int *x_start, const int *x_width, int x_len,
 			gapwidth = start_j - gapstart;
 			if (gapwidth >= 1) {
 				/* Append to 'out_ranges'. */
-				_RangeAE_insert_at(out_ranges,
+				IntPairAE_insert_at(out_ranges,
 					out_len,
 					gapstart, gapwidth);
 				out_len++;
@@ -365,7 +367,7 @@ static int gaps_ranges(const int *x_start, const int *x_width, int x_len,
 		gapstart = max_end + 1;
 		gapwidth = restrict_end - max_end;
 		/* Append to 'out_ranges'. */
-		_RangeAE_insert_at(out_ranges,
+		IntPairAE_insert_at(out_ranges,
 			out_len,
 			gapstart, gapwidth);
 		out_len++;
@@ -379,14 +381,14 @@ SEXP IRanges_gaps(SEXP x_start, SEXP x_width, SEXP start, SEXP end)
 	int x_len;
 	const int *x_start_p, *x_width_p;
 	SEXP ans, ans_names;
-	RangeAE out_ranges;
+	IntPairAE out_ranges;
 	IntAE order_buf;
 
-	x_len = _check_integer_pairs(x_start, x_width,
-				     &x_start_p, &x_width_p,
-				     "start(x)", "width(x)");
-	out_ranges = _new_RangeAE(0, 0);
-	order_buf = _new_IntAE(x_len, 0, 0);
+	x_len = check_integer_pairs(x_start, x_width,
+				    &x_start_p, &x_width_p,
+				    "start(x)", "width(x)");
+	out_ranges = new_IntPairAE(0, 0);
+	order_buf = new_IntAE(x_len, 0, 0);
 	gaps_ranges(x_start_p, x_width_p, x_len,
 		INTEGER(start)[0], INTEGER(end)[0],
 		order_buf.elts, &out_ranges);
@@ -397,8 +399,8 @@ SEXP IRanges_gaps(SEXP x_start, SEXP x_width, SEXP start, SEXP end)
 	SET_STRING_ELT(ans_names, 1, mkChar("width"));
 	SET_NAMES(ans, ans_names);
 	UNPROTECT(1);
-	SET_VECTOR_ELT(ans, 0, _new_INTEGER_from_IntAE(&(out_ranges.start)));
-	SET_VECTOR_ELT(ans, 1, _new_INTEGER_from_IntAE(&(out_ranges.width)));
+	SET_VECTOR_ELT(ans, 0, new_INTEGER_from_IntAE(&(out_ranges.a)));
+	SET_VECTOR_ELT(ans, 1, new_INTEGER_from_IntAE(&(out_ranges.b)));
 	UNPROTECT(1);
 	return ans;
 }
@@ -412,14 +414,14 @@ SEXP CompressedIRangesList_gaps(SEXP x, SEXP start, SEXP end)
 	IRanges_holder ir_holder;
 	int x_len, in_len_max, start_len, *start_elt, *end_elt, i;
 	IntAE order_buf;
-	RangeAE in_ranges, out_ranges;
+	IntPairAE in_ranges, out_ranges;
 
 	x_holder = _hold_CompressedIRangesList(x);
 	x_len = _get_length_from_CompressedIRangesList_holder(&x_holder);
 	in_len_max = get_elt_from_CompressedIRangesList_holderlens_max(&x_holder);
-	order_buf = _new_IntAE(in_len_max, 0, 0);
-	in_ranges = _new_RangeAE(0, 0);
-	out_ranges = _new_RangeAE(0, 0);
+	order_buf = new_IntAE(in_len_max, 0, 0);
+	in_ranges = new_IntPairAE(0, 0);
+	out_ranges = new_IntPairAE(0, 0);
 	start_len = LENGTH(start);
 	if ((start_len != 1 && start_len != x_len) || start_len != LENGTH(end))
 		error("'start' and 'end' should both be integer vectors "
@@ -429,25 +431,25 @@ SEXP CompressedIRangesList_gaps(SEXP x, SEXP start, SEXP end)
 	end_elt = INTEGER(end);
 	for (i = 0; i < x_len; i++) {
 		ir_holder = _get_elt_from_CompressedIRangesList_holder(&x_holder, i);
-		_RangeAE_set_nelt(&in_ranges, 0);
-		append_IRanges_holder_to_RangeAE(&in_ranges, &ir_holder);
-		gaps_ranges(in_ranges.start.elts, in_ranges.width.elts,
-			_RangeAE_get_nelt(&in_ranges),
+		IntPairAE_set_nelt(&in_ranges, 0);
+		append_IRanges_holder_to_IntPairAE(&in_ranges, &ir_holder);
+		gaps_ranges(in_ranges.a.elts, in_ranges.b.elts,
+			IntPairAE_get_nelt(&in_ranges),
 			*start_elt, *end_elt,
 			order_buf.elts, &out_ranges);
-		INTEGER(ans_partitioning_end)[i] = _RangeAE_get_nelt(&out_ranges);
+		INTEGER(ans_partitioning_end)[i] = IntPairAE_get_nelt(&out_ranges);
 		if (start_len != 1) {
 			start_elt++;
 			end_elt++;
 		}
 	}
-	PROTECT(ans_unlistData = _new_IRanges_from_RangeAE("IRanges",
+	PROTECT(ans_unlistData = _new_IRanges_from_IntPairAE("IRanges",
 			&out_ranges));
 	PROTECT(ans_names = duplicate(_get_CompressedList_names(x)));
 	PROTECT(ans_partitioning = _new_PartitioningByEnd(
 			"PartitioningByEnd",
 			ans_partitioning_end, ans_names));
-	PROTECT(ans = _new_CompressedList(_get_classname(x),
+	PROTECT(ans = _new_CompressedList(get_classname(x),
 			ans_unlistData, ans_partitioning));
 	UNPROTECT(5);
 	return ans;
@@ -464,17 +466,17 @@ SEXP CompressedIRangesList_gaps(SEXP x, SEXP start, SEXP end)
 SEXP Ranges_disjointBins(SEXP x_start, SEXP x_width)
 {
   SEXP ans;
-  IntAE bin_ends = _new_IntAE(128, 0, 0);
+  IntAE bin_ends = new_IntAE(128, 0, 0);
 
   PROTECT(ans = NEW_INTEGER(length(x_start)));
 
   for (int i = 0; i < length(x_start); i++) {
     // find a bin, starting at first
     int j = 0, end = INTEGER(x_start)[i] + INTEGER(x_width)[i] - 1;
-    for (; j < _IntAE_get_nelt(&bin_ends) && bin_ends.elts[j] >= INTEGER(x_start)[i]; j++);
+    for (; j < IntAE_get_nelt(&bin_ends) && bin_ends.elts[j] >= INTEGER(x_start)[i]; j++);
     // remember when this bin will be open
-    if (j == _IntAE_get_nelt(&bin_ends))
-      _IntAE_append(&bin_ends, &end, 1);
+    if (j == IntAE_get_nelt(&bin_ends))
+      IntAE_append(&bin_ends, &end, 1);
     else bin_ends.elts[j] = end;
     // store the bin for this range
     INTEGER(ans)[i] = j + 1;
